# -*- coding: utf-8 -*-

import subprocess


def get_terminal_size():
    r = subprocess.check_output("stty size", shell=True)
    try:
        size = str(r, encoding="utf8").strip().split(' ')
    except:
        # py2
        size = str(r).strip().split(' ')
    return int(size[0]), int(size[1])


def is_ascii(c):
    return 255 >= ord(c) >= 0



import threading
import time

from terminal_layout import *
from terminal_layout.extensions.progress.progress import Progress, SuffixStyle
from terminal_layout.helper.class_helper import instance_variables


class InfixChoices:
    style1 = ['-', '\\', '|', '/']
    style2 = ["◰", "◳", "◲", "◱"]
    style3 = ["◐", "◓", "◑", "◒"]
    style4 = [".", "o", "O", "°", "O", "o", "."]
    style5 = ['▁', '▂', '▃', '▅', '▆', '▇', ]
    style6 = ["ဝ", "၀"]
    style7 = ['⣾', '⣷', '⣯', '⣟', '⡿', '⢿', '⣻', '⣽']
    style8 = ['◷', '◶', '◵', '◴']
    style9 = ['( ◐ ω ◐ )', '( ◓ ω ◓ )', '( ◑ ω ◑ )', '( ◒ ω ◒ )', ]
    style10 = ["( ^ . ^ )", "( ^ o ^ )", "( ^ O ^ )", "( ^ o ^ )"]


class Loading(Progress):
    ctl = None

    @instance_variables
    def __init__(self, prefix, max, refresh_time=0.2, delimiter=None, infix=None,
                 suffix_style=SuffixStyle.percent):
        """


        :param prefix: 
        :type prefix: str
        :param max: 
        :type max: int
        :param suffix_style: 
        :type suffix_style: SuffixStyle
        """
        if infix is None:
            self.infix = InfixChoices.style7
        if delimiter is None:
            self.delimiter = [' ', ' ']
        self.infix_index = 0
        self.current_progress = 0
        self.lock = threading.Lock()
        self.update_infix_thread = threading.Thread(
            target=self.update_infix,
            args=(self,)
        )
        self.update_infix_thread.daemon = True
        self.stop_flag = False

    @staticmethod
    def update_infix(loading):
        def run():
            progress_view = loading.ctl.find_view_by_id('progress')
            loading.infix_index += 1
            loading.infix_index %= len(loading.infix)
            progress_view.set_text(loading.infix[loading.infix_index])

        while True:
            if loading.stop_flag:
                break
            loading.run_with_lock(run)
            time.sleep(loading.refresh_time)

    def run_with_lock(self, func, args=None):
        if args == None:
            args = []
        self.lock.acquire()
        func(*args)
        self.lock.release()

    def add_progress(self, num):
        self.run_with_lock(super().add_progress, [num])

    def set_progress(self, num):
        self.run_with_lock(super().set_progress, [num])

    def update(self):

        self.ctl.find_view_by_id('suffix').set_text(self.get_suffix())

    def start(self):
        prefix_width = len(self.prefix)
        if self.suffix_style == SuffixStyle.percent:
            suffix_width = 4
        else:
            suffix_width = len(str(self.max)) * 2 + 1

        self.ctl = LayoutCtl.quick(TableLayout,
                                   [[TextView('prefix', self.prefix, width=prefix_width),
                                     TextView('', self.delimiter[0]),
                                     TextView('progress', ''),
                                     TextView('', self.delimiter[1]),
                                     TextView('suffix', self.get_suffix(), width=suffix_width, gravity=Gravity.right)]])

        self.update()
        self.stop_flag = False
        self.update_infix_thread.start()
        self.ctl.draw()

    def stop(self):
        self.stop_flag = True
        if self.ctl:
            self.ctl.stop()
            self.ctl = None

    def is_finished(self):
        return self.current_progress >= self.max

    def __enter__(self):
        self.start()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.stop()



import sys
import inspect

if sys.version_info >= (3, 0):
    from functools import wraps


    def instance_variables(f):
        sig = inspect.signature(f)

        @wraps(f)
        def wrapper(self, *args, **kwargs):
            values = sig.bind(self, *args, **kwargs)
            for k, p in sig.parameters.items():
                if k != 'self':
                    if k in values.arguments:
                        val = values.arguments[k]
                        if p.kind in (inspect.Parameter.POSITIONAL_OR_KEYWORD, inspect.Parameter.KEYWORD_ONLY):
                            setattr(self, k, val)
                        elif p.kind == inspect.Parameter.VAR_KEYWORD:
                            for k, v in values.arguments[k].items():
                                setattr(self, k, v)
                    else:
                        setattr(self, k, p.default)

        return wrapper
else:
    def instance_variables(func):
        def return_func(*args, **kwargs):
            self_var = args[0]

            arg_spec = inspect.getargspec(func)
            argument_names = arg_spec[0][1:]
            defaults = arg_spec[3]
            if defaults is not None:
                default_arg_dict = dict(zip(reversed(argument_names), reversed(defaults)))
                self_var.__dict__.update(default_arg_dict)

            arg_dict = dict(zip(argument_names, args[1:]))
            self_var.__dict__.update(arg_dict)

            valid_keywords = set(kwargs) & set(argument_names)
            kwarg_dict = {k: kwargs[k] for k in valid_keywords}
            self_var.__dict__.update(kwarg_dict)

            func(*args, **kwargs)

        return return_func


def instance_variables(func):
    def return_func(*args, **kwargs):
        self_var = args[0]

        arg_spec = inspect.getargspec(func)
        argument_names = arg_spec[0][1:]
        defaults = arg_spec[3]
        if defaults is not None:
            default_arg_dict = dict(zip(reversed(argument_names), reversed(defaults)))
            self_var.__dict__.update(default_arg_dict)

        arg_dict = dict(zip(argument_names, args[1:]))
        self_var.__dict__.update(arg_dict)

        valid_keywords = set(kwargs) & set(argument_names)
        kwarg_dict = {k: kwargs[k] for k in valid_keywords}
        self_var.__dict__.update(kwarg_dict)

        func(*args, **kwargs)

    return return_func



# -*- coding: utf-8 -*-
# -*- coding: utf-8 -*-
from terminal_layout import *

print('\n[ex]style support linux, osx\n')

ctl = LayoutCtl.quick(TableLayout,
                      [[TextView('', 'style = ex_bold', style=Style.ex_bold)],
                       [TextView('', 'style = ex_dim', style=Style.ex_dim)],
                       [TextView('', 'style = ex_underlined', style=Style.ex_underlined)],
                       [TextView('', 'style = ex_blink', style=Style.ex_blink)],
                       [TextView('', 'style = ex_reverse', style=Style.ex_reverse)],
                       ])

ctl.draw(auto_re_draw=False)



# -*- coding: utf-8 -*-
# Initially taken from:
# http://code.activestate.com/recipes/134892/#c9
# Thanks to Stephen Chappell
import msvcrt

from terminal_layout.readkey.key import Key

xlate_dict = {
    8: Key.BACKSPACE.code,
    13: Key.ENTER.code,
    27: Key.ESC.code,
    15104: Key.F1.code,
    15360: Key.F2.code,
    15616: Key.F3.code,
    15872: Key.F4.code,
    16128: Key.F5.code,
    16384: Key.F6.code,
    16640: Key.F7.code,
    16896: Key.F8.code,

    18656: Key.UP.code,
    20704: Key.DOWN.code,
    19424: Key.LEFT.code,
    19936: Key.RIGHT.code,
}


def readkey():
    while True:
        if msvcrt.kbhit():
            ch = msvcrt.getch()
            a = ord(ch)
            if a == 0 or a == 224:
                b = ord(msvcrt.getch())
                x = a + (b * 256)
                return xlate_dict.get(x, None)
            else:
                return xlate_dict.get(a, ch.decode())



# -*- coding: utf-8 -*-
# -*- coding: utf-8 -*-
# -*- coding: utf-8 -*-
from terminal_layout import *

ctl = LayoutCtl.quick(TableLayout,
                      [
                          [TextView('', 'visibility = visible ', width=25, back=Back.green),
                           TextView('', ' hhhh ', visibility=Visibility.visible, back=Back.lightblack),
                           TextView('', ' end ', back=Back.red)],
                          [TextView('', 'visibility = invisible ', width=25, back=Back.green),
                           TextView('', ' hhhh ', visibility=Visibility.invisible, back=Back.lightblack),
                           TextView('', ' end ', back=Back.red)],
                          [TextView('', 'visibility = gone ', width=25, back=Back.green),
                           TextView('', ' hhhh ', visibility=Visibility.gone, back=Back.lightblack),
                           TextView('', ' end ', back=Back.red)],

                      ])

ctl.draw(auto_re_draw=False)



import logging

logger = logging.getLogger(__name__)
logger.setLevel(level=logging.DEBUG)
# handler = logging.FileHandler("terminal_layout.log")
# handler.setLevel(logging.DEBUG)
# formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')
# handler.setFormatter(formatter)
# logger.addHandler(handler)



# -*- coding: utf-8 -*-


class Width(object):
    wrap = 'wrap'
    fill = 'fill'


class Visibility(object):
    visible = 'visible'
    invisible = 'invisible'
    gone = 'gone'


class Gravity(object):
    left = 'left'
    center = 'center'
    right = 'right'


################ linearlayout ###############3

class Orientation(object):
    # 水平
    horizon = 'horizon'
    # 垂直
    vertical = 'vertical'




from terminal_layout.ansi.fore import Fore as _Fore
from terminal_layout.ansi.back import Back as _Back
from terminal_layout.ansi.style import Style as _Style
from colorama import Cursor as _Cursor
from colorama import init as _init
from colorama.ansi import clear_line as _clear_line
from colorama.ansi import clear_screen as _clear_screen

Fore = _Fore
Back = _Back
Style = _Style
Cursor = _Cursor

term_init = _init
clear_line = _clear_line
clear_screen = _clear_screen



class KeyPressEvent(object):
    def __init__(self, k, t):
        """
        
        :param k:
        :type k: KeyInfo
        :param t:
        :type t:
        """
        self.key = k
        self.time = t

    def __str__(self):
        return 'KeyPressEvent(key=<%s>, time=%s)' % (self.key.name, self.time)



# -*- coding: utf-8 -*-

from terminal_layout import *

ctl = LayoutCtl.quick(TableLayout,
                      [
                          [TextView('', 'gravity = left', width=25, gravity=Gravity.left, back=Back.cyan)],
                          [TextView('', 'gravity = center', width=25, gravity=Gravity.center, back=Back.green)],
                          [TextView('', 'gravity = right', width=25, gravity=Gravity.right, back=Back.magenta)],
                      ])

ctl.get_layout().set_width(50)

ctl.draw(auto_re_draw=False)



# -*- coding: utf-8 -*-
from terminal_layout.ansi.font import Font


class Fore(object):
    black = Font('black', 'fore', 30)
    red = Font('red', 'fore', 31)
    green = Font('green', 'fore', 32)
    yellow = Font('yellow', 'fore', 33)
    blue = Font('blue', 'fore', 34)
    magenta = Font('magenta', 'fore', 35)
    cyan = Font('cyan', 'fore', 36)
    white = Font('white', 'fore', 37)
    reset = Font('reset', 'fore', 39)

    # these are fairly well supported, but not part of the standard.
    lightblack = Font('lightblack', 'fore', 90)
    lightred = Font('lightred', 'fore', 91)
    lightgreen = Font('lightgreen', 'fore', 92)
    lightyellow = Font('lightyellow', 'fore', 93)
    lightblue = Font('lightblue', 'fore', 94)
    lightmagenta = Font('lightmagenta', 'fore', 95)
    lightcyan = Font('lightcyan', 'fore', 96)
    lightwhite = Font('lightwhite', 'fore', 97)

    # ex_xx only support linux, mac

    ex_black = Font('ex_black', 'fore')
    ex_red = Font('ex_red', 'fore')
    ex_green = Font('ex_green', 'fore')
    ex_yellow = Font('ex_yellow', 'fore')
    ex_blue = Font('ex_blue', 'fore')
    ex_magenta = Font('ex_magenta', 'fore')
    ex_cyan = Font('ex_cyan', 'fore')
    ex_light_gray = Font('ex_light_gray', 'fore')
    ex_dark_gray = Font('ex_dark_gray', 'fore')
    ex_light_red = Font('ex_light_red', 'fore')
    ex_light_green = Font('ex_light_green', 'fore')
    ex_light_yellow = Font('ex_light_yellow', 'fore')
    ex_light_blue = Font('ex_light_blue', 'fore')
    ex_light_magenta = Font('ex_light_magenta', 'fore')
    ex_light_cyan = Font('ex_light_cyan', 'fore')
    ex_white = Font('ex_white', 'fore')
    ex_grey_0 = Font('ex_grey_0', 'fore')
    ex_navy_blue = Font('ex_navy_blue', 'fore')
    ex_dark_blue = Font('ex_dark_blue', 'fore')
    ex_blue_3a = Font('ex_blue_3a', 'fore')
    ex_blue_3b = Font('ex_blue_3b', 'fore')
    ex_blue_1 = Font('ex_blue_1', 'fore')
    ex_dark_green = Font('ex_dark_green', 'fore')
    ex_deep_sky_blue_4a = Font('ex_deep_sky_blue_4a', 'fore')
    ex_deep_sky_blue_4b = Font('ex_deep_sky_blue_4b', 'fore')
    ex_deep_sky_blue_4c = Font('ex_deep_sky_blue_4c', 'fore')
    ex_dodger_blue_3 = Font('ex_dodger_blue_3', 'fore')
    ex_dodger_blue_2 = Font('ex_dodger_blue_2', 'fore')
    ex_green_4 = Font('ex_green_4', 'fore')
    ex_spring_green_4 = Font('ex_spring_green_4', 'fore')
    ex_turquoise_4 = Font('ex_turquoise_4', 'fore')
    ex_deep_sky_blue_3a = Font('ex_deep_sky_blue_3a', 'fore')
    ex_deep_sky_blue_3b = Font('ex_deep_sky_blue_3b', 'fore')
    ex_dodger_blue_1 = Font('ex_dodger_blue_1', 'fore')
    ex_green_3a = Font('ex_green_3a', 'fore')
    ex_spring_green_3a = Font('ex_spring_green_3a', 'fore')
    ex_dark_cyan = Font('ex_dark_cyan', 'fore')
    ex_light_sea_green = Font('ex_light_sea_green', 'fore')
    ex_deep_sky_blue_2 = Font('ex_deep_sky_blue_2', 'fore')
    ex_deep_sky_blue_1 = Font('ex_deep_sky_blue_1', 'fore')
    ex_green_3b = Font('ex_green_3b', 'fore')
    ex_spring_green_3b = Font('ex_spring_green_3b', 'fore')
    ex_spring_green_2a = Font('ex_spring_green_2a', 'fore')
    ex_cyan_3 = Font('ex_cyan_3', 'fore')
    ex_dark_turquoise = Font('ex_dark_turquoise', 'fore')
    ex_turquoise_2 = Font('ex_turquoise_2', 'fore')
    ex_green_1 = Font('ex_green_1', 'fore')
    ex_spring_green_2b = Font('ex_spring_green_2b', 'fore')
    ex_spring_green_1 = Font('ex_spring_green_1', 'fore')
    ex_medium_spring_green = Font('ex_medium_spring_green', 'fore')
    ex_cyan_2 = Font('ex_cyan_2', 'fore')
    ex_cyan_1 = Font('ex_cyan_1', 'fore')
    ex_dark_red_1 = Font('ex_dark_red_1', 'fore')
    ex_deep_pink_4a = Font('ex_deep_pink_4a', 'fore')
    ex_purple_4a = Font('ex_purple_4a', 'fore')
    ex_purple_4b = Font('ex_purple_4b', 'fore')
    ex_purple_3 = Font('ex_purple_3', 'fore')
    ex_blue_violet = Font('ex_blue_violet', 'fore')
    ex_orange_4a = Font('ex_orange_4a', 'fore')
    ex_grey_37 = Font('ex_grey_37', 'fore')
    ex_medium_purple_4 = Font('ex_medium_purple_4', 'fore')
    ex_slate_blue_3a = Font('ex_slate_blue_3a', 'fore')
    ex_slate_blue_3b = Font('ex_slate_blue_3b', 'fore')
    ex_royal_blue_1 = Font('ex_royal_blue_1', 'fore')
    ex_chartreuse_4 = Font('ex_chartreuse_4', 'fore')
    ex_dark_sea_green_4a = Font('ex_dark_sea_green_4a', 'fore')
    ex_pale_turquoise_4 = Font('ex_pale_turquoise_4', 'fore')
    ex_steel_blue = Font('ex_steel_blue', 'fore')
    ex_steel_blue_3 = Font('ex_steel_blue_3', 'fore')
    ex_cornflower_blue = Font('ex_cornflower_blue', 'fore')
    ex_chartreuse_3a = Font('ex_chartreuse_3a', 'fore')
    ex_dark_sea_green_4b = Font('ex_dark_sea_green_4b', 'fore')
    ex_cadet_blue_2 = Font('ex_cadet_blue_2', 'fore')
    ex_cadet_blue_1 = Font('ex_cadet_blue_1', 'fore')
    ex_sky_blue_3 = Font('ex_sky_blue_3', 'fore')
    ex_steel_blue_1a = Font('ex_steel_blue_1a', 'fore')
    ex_chartreuse_3b = Font('ex_chartreuse_3b', 'fore')
    ex_pale_green_3a = Font('ex_pale_green_3a', 'fore')
    ex_sea_green_3 = Font('ex_sea_green_3', 'fore')
    ex_aquamarine_3 = Font('ex_aquamarine_3', 'fore')
    ex_medium_turquoise = Font('ex_medium_turquoise', 'fore')
    ex_steel_blue_1b = Font('ex_steel_blue_1b', 'fore')
    ex_chartreuse_2a = Font('ex_chartreuse_2a', 'fore')
    ex_sea_green_2 = Font('ex_sea_green_2', 'fore')
    ex_sea_green_1a = Font('ex_sea_green_1a', 'fore')
    ex_sea_green_1b = Font('ex_sea_green_1b', 'fore')
    ex_aquamarine_1a = Font('ex_aquamarine_1a', 'fore')
    ex_dark_slate_gray_2 = Font('ex_dark_slate_gray_2', 'fore')
    ex_dark_red_2 = Font('ex_dark_red_2', 'fore')
    ex_deep_pink_4b = Font('ex_deep_pink_4b', 'fore')
    ex_dark_magenta_1 = Font('ex_dark_magenta_1', 'fore')
    ex_dark_magenta_2 = Font('ex_dark_magenta_2', 'fore')
    ex_dark_violet_1a = Font('ex_dark_violet_1a', 'fore')
    ex_purple_1a = Font('ex_purple_1a', 'fore')
    ex_orange_4b = Font('ex_orange_4b', 'fore')
    ex_light_pink_4 = Font('ex_light_pink_4', 'fore')
    ex_plum_4 = Font('ex_plum_4', 'fore')
    ex_medium_purple_3a = Font('ex_medium_purple_3a', 'fore')
    ex_medium_purple_3b = Font('ex_medium_purple_3b', 'fore')
    ex_slate_blue_1 = Font('ex_slate_blue_1', 'fore')
    ex_yellow_4a = Font('ex_yellow_4a', 'fore')
    ex_wheat_4 = Font('ex_wheat_4', 'fore')
    ex_grey_53 = Font('ex_grey_53', 'fore')
    ex_light_slate_grey = Font('ex_light_slate_grey', 'fore')
    ex_medium_purple = Font('ex_medium_purple', 'fore')
    ex_light_slate_blue = Font('ex_light_slate_blue', 'fore')
    ex_yellow_4b = Font('ex_yellow_4b', 'fore')
    ex_dark_olive_green_3a = Font('ex_dark_olive_green_3a', 'fore')
    ex_dark_green_sea = Font('ex_dark_green_sea', 'fore')
    ex_light_sky_blue_3a = Font('ex_light_sky_blue_3a', 'fore')
    ex_light_sky_blue_3b = Font('ex_light_sky_blue_3b', 'fore')
    ex_sky_blue_2 = Font('ex_sky_blue_2', 'fore')
    ex_chartreuse_2b = Font('ex_chartreuse_2b', 'fore')
    ex_dark_olive_green_3b = Font('ex_dark_olive_green_3b', 'fore')
    ex_pale_green_3b = Font('ex_pale_green_3b', 'fore')
    ex_dark_sea_green_3a = Font('ex_dark_sea_green_3a', 'fore')
    ex_dark_slate_gray_3 = Font('ex_dark_slate_gray_3', 'fore')
    ex_sky_blue_1 = Font('ex_sky_blue_1', 'fore')
    ex_chartreuse_1 = Font('ex_chartreuse_1', 'fore')
    ex_light_green_2 = Font('ex_light_green_2', 'fore')
    ex_light_green_3 = Font('ex_light_green_3', 'fore')
    ex_pale_green_1a = Font('ex_pale_green_1a', 'fore')
    ex_aquamarine_1b = Font('ex_aquamarine_1b', 'fore')
    ex_dark_slate_gray_1 = Font('ex_dark_slate_gray_1', 'fore')
    ex_red_3a = Font('ex_red_3a', 'fore')
    ex_deep_pink_4c = Font('ex_deep_pink_4c', 'fore')
    ex_medium_violet_red = Font('ex_medium_violet_red', 'fore')
    ex_magenta_3a = Font('ex_magenta_3a', 'fore')
    ex_dark_violet_1b = Font('ex_dark_violet_1b', 'fore')
    ex_purple_1b = Font('ex_purple_1b', 'fore')
    ex_dark_orange_3a = Font('ex_dark_orange_3a', 'fore')
    ex_indian_red_1a = Font('ex_indian_red_1a', 'fore')
    ex_hot_pink_3a = Font('ex_hot_pink_3a', 'fore')
    ex_medium_orchid_3 = Font('ex_medium_orchid_3', 'fore')
    ex_medium_orchid = Font('ex_medium_orchid', 'fore')
    ex_medium_purple_2a = Font('ex_medium_purple_2a', 'fore')
    ex_dark_goldenrod = Font('ex_dark_goldenrod', 'fore')
    ex_light_salmon_3a = Font('ex_light_salmon_3a', 'fore')
    ex_rosy_brown = Font('ex_rosy_brown', 'fore')
    ex_grey_63 = Font('ex_grey_63', 'fore')
    ex_medium_purple_2b = Font('ex_medium_purple_2b', 'fore')
    ex_medium_purple_1 = Font('ex_medium_purple_1', 'fore')
    ex_gold_3a = Font('ex_gold_3a', 'fore')
    ex_dark_khaki = Font('ex_dark_khaki', 'fore')
    ex_navajo_white_3 = Font('ex_navajo_white_3', 'fore')
    ex_grey_69 = Font('ex_grey_69', 'fore')
    ex_light_steel_blue_3 = Font('ex_light_steel_blue_3', 'fore')
    ex_light_steel_blue = Font('ex_light_steel_blue', 'fore')
    ex_yellow_3a = Font('ex_yellow_3a', 'fore')
    ex_dark_olive_green_3 = Font('ex_dark_olive_green_3', 'fore')
    ex_dark_sea_green_3b = Font('ex_dark_sea_green_3b', 'fore')
    ex_dark_sea_green_2 = Font('ex_dark_sea_green_2', 'fore')
    ex_light_cyan_3 = Font('ex_light_cyan_3', 'fore')
    ex_light_sky_blue_1 = Font('ex_light_sky_blue_1', 'fore')
    ex_green_yellow = Font('ex_green_yellow', 'fore')
    ex_dark_olive_green_2 = Font('ex_dark_olive_green_2', 'fore')
    ex_pale_green_1b = Font('ex_pale_green_1b', 'fore')
    ex_dark_sea_green_5b = Font('ex_dark_sea_green_5b', 'fore')
    ex_dark_sea_green_5a = Font('ex_dark_sea_green_5a', 'fore')
    ex_pale_turquoise_1 = Font('ex_pale_turquoise_1', 'fore')
    ex_red_3b = Font('ex_red_3b', 'fore')
    ex_deep_pink_3a = Font('ex_deep_pink_3a', 'fore')
    ex_deep_pink_3b = Font('ex_deep_pink_3b', 'fore')
    ex_magenta_3b = Font('ex_magenta_3b', 'fore')
    ex_magenta_3c = Font('ex_magenta_3c', 'fore')
    ex_magenta_2a = Font('ex_magenta_2a', 'fore')
    ex_dark_orange_3b = Font('ex_dark_orange_3b', 'fore')
    ex_indian_red_1b = Font('ex_indian_red_1b', 'fore')
    ex_hot_pink_3b = Font('ex_hot_pink_3b', 'fore')
    ex_hot_pink_2 = Font('ex_hot_pink_2', 'fore')
    ex_orchid = Font('ex_orchid', 'fore')
    ex_medium_orchid_1a = Font('ex_medium_orchid_1a', 'fore')
    ex_orange_3 = Font('ex_orange_3', 'fore')
    ex_light_salmon_3b = Font('ex_light_salmon_3b', 'fore')
    ex_light_pink_3 = Font('ex_light_pink_3', 'fore')
    ex_pink_3 = Font('ex_pink_3', 'fore')
    ex_plum_3 = Font('ex_plum_3', 'fore')
    ex_violet = Font('ex_violet', 'fore')
    ex_gold_3b = Font('ex_gold_3b', 'fore')
    ex_light_goldenrod_3 = Font('ex_light_goldenrod_3', 'fore')
    ex_tan = Font('ex_tan', 'fore')
    ex_misty_rose_3 = Font('ex_misty_rose_3', 'fore')
    ex_thistle_3 = Font('ex_thistle_3', 'fore')
    ex_plum_2 = Font('ex_plum_2', 'fore')
    ex_yellow_3b = Font('ex_yellow_3b', 'fore')
    ex_khaki_3 = Font('ex_khaki_3', 'fore')
    ex_light_goldenrod_2a = Font('ex_light_goldenrod_2a', 'fore')
    ex_light_yellow_3 = Font('ex_light_yellow_3', 'fore')
    ex_grey_84 = Font('ex_grey_84', 'fore')
    ex_light_steel_blue_1 = Font('ex_light_steel_blue_1', 'fore')
    ex_yellow_2 = Font('ex_yellow_2', 'fore')
    ex_dark_olive_green_1a = Font('ex_dark_olive_green_1a', 'fore')
    ex_dark_olive_green_1b = Font('ex_dark_olive_green_1b', 'fore')
    ex_dark_sea_green_1 = Font('ex_dark_sea_green_1', 'fore')
    ex_honeydew_2 = Font('ex_honeydew_2', 'fore')
    ex_light_cyan_1 = Font('ex_light_cyan_1', 'fore')
    ex_red_1 = Font('ex_red_1', 'fore')
    ex_deep_pink_2 = Font('ex_deep_pink_2', 'fore')
    ex_deep_pink_1a = Font('ex_deep_pink_1a', 'fore')
    ex_deep_pink_1b = Font('ex_deep_pink_1b', 'fore')
    ex_magenta_2b = Font('ex_magenta_2b', 'fore')
    ex_magenta_1 = Font('ex_magenta_1', 'fore')
    ex_orange_red_1 = Font('ex_orange_red_1', 'fore')
    ex_indian_red_1c = Font('ex_indian_red_1c', 'fore')
    ex_indian_red_1d = Font('ex_indian_red_1d', 'fore')
    ex_hot_pink_1a = Font('ex_hot_pink_1a', 'fore')
    ex_hot_pink_1b = Font('ex_hot_pink_1b', 'fore')
    ex_medium_orchid_1b = Font('ex_medium_orchid_1b', 'fore')
    ex_dark_orange = Font('ex_dark_orange', 'fore')
    ex_salmon_1 = Font('ex_salmon_1', 'fore')
    ex_light_coral = Font('ex_light_coral', 'fore')
    ex_pale_violet_red_1 = Font('ex_pale_violet_red_1', 'fore')
    ex_orchid_2 = Font('ex_orchid_2', 'fore')
    ex_orchid_1 = Font('ex_orchid_1', 'fore')
    ex_orange_1 = Font('ex_orange_1', 'fore')
    ex_sandy_brown = Font('ex_sandy_brown', 'fore')
    ex_light_salmon_1 = Font('ex_light_salmon_1', 'fore')
    ex_light_pink_1 = Font('ex_light_pink_1', 'fore')
    ex_pink_1 = Font('ex_pink_1', 'fore')
    ex_plum_1 = Font('ex_plum_1', 'fore')
    ex_gold_1 = Font('ex_gold_1', 'fore')
    ex_light_goldenrod_2b = Font('ex_light_goldenrod_2b', 'fore')
    ex_light_goldenrod_2c = Font('ex_light_goldenrod_2c', 'fore')
    ex_navajo_white_1 = Font('ex_navajo_white_1', 'fore')
    ex_misty_rose1 = Font('ex_misty_rose1', 'fore')
    ex_thistle_1 = Font('ex_thistle_1', 'fore')
    ex_yellow_1 = Font('ex_yellow_1', 'fore')
    ex_light_goldenrod_1 = Font('ex_light_goldenrod_1', 'fore')
    ex_khaki_1 = Font('ex_khaki_1', 'fore')
    ex_wheat_1 = Font('ex_wheat_1', 'fore')
    ex_cornsilk_1 = Font('ex_cornsilk_1', 'fore')
    ex_grey_100 = Font('ex_grey_100', 'fore')
    ex_grey_3 = Font('ex_grey_3', 'fore')
    ex_grey_7 = Font('ex_grey_7', 'fore')
    ex_grey_11 = Font('ex_grey_11', 'fore')
    ex_grey_15 = Font('ex_grey_15', 'fore')
    ex_grey_19 = Font('ex_grey_19', 'fore')
    ex_grey_23 = Font('ex_grey_23', 'fore')
    ex_grey_27 = Font('ex_grey_27', 'fore')
    ex_grey_30 = Font('ex_grey_30', 'fore')
    ex_grey_35 = Font('ex_grey_35', 'fore')
    ex_grey_39 = Font('ex_grey_39', 'fore')
    ex_grey_42 = Font('ex_grey_42', 'fore')
    ex_grey_46 = Font('ex_grey_46', 'fore')
    ex_grey_50 = Font('ex_grey_50', 'fore')
    ex_grey_54 = Font('ex_grey_54', 'fore')
    ex_grey_58 = Font('ex_grey_58', 'fore')
    ex_grey_62 = Font('ex_grey_62', 'fore')
    ex_grey_66 = Font('ex_grey_66', 'fore')
    ex_grey_70 = Font('ex_grey_70', 'fore')
    ex_grey_74 = Font('ex_grey_74', 'fore')
    ex_grey_78 = Font('ex_grey_78', 'fore')
    ex_grey_82 = Font('ex_grey_82', 'fore')
    ex_grey_85 = Font('ex_grey_85', 'fore')
    ex_grey_89 = Font('ex_grey_89', 'fore')
    ex_grey_93 = Font('ex_grey_93', 'fore')





# -*- coding: utf-8 -*-

import sys

from terminal_layout.ansi import Cursor, clear_line, Style
from terminal_layout.view.base import View
from terminal_layout.view.params import Visibility, Gravity, Orientation, Width
from terminal_layout.view.text_view import TextView


class TableRow(View):
    __slots__ = ('back', 'child_width')

    def __init__(self, id, width=Width.fill, height=1, back=None, visibility=Visibility.visible, gravity=Gravity.left):
        """

        :param id:
        :param width:
        :param height: no used

        :type id:str
        :type width:
        :type height:int
        :type visibility:str
        :type gravity:str
        """

        super(TableRow, self).__init__(id, width, height, visibility, gravity)
        self.back = back or ''
        self.data = []  # type:list[TextView]

    @classmethod
    def quick_init(cls, id, data, width=Width.fill, height=1, back=None, visibility=Visibility.visible,
                   gravity=Gravity.left):
        """

        :param id:
        :param data:
        :param width:
        :param height:
        :param back:
        :param visibility:
        :param gravity:
        :return:

        :rtype:TableRow
        """

        row = cls(id, width, height, back, visibility, gravity)
        row.data = data
        return row

    def add_view(self, v):
        if not isinstance(v, TextView):
            raise TypeError('only support add TextView')
        self.data.append(v)

    def add_views(self, *views):
        self.data += views

    def add_view_list(self, views):
        self.data += views

    def draw(self):
        if self.visibility == Visibility.visible:
            left = ''
            right = ''
            if self.real_width > self.child_width:
                if self.gravity == Gravity.left:
                    right = ' ' * (self.real_width - self.child_width)
                elif self.gravity == Gravity.right:
                    left = ' ' * (self.real_width - self.child_width)
                elif self.gravity == Gravity.center:
                    p = (self.real_width - self.child_width) / 2.0
                    left = ' ' * int(p)
                    right = ' ' * (int(p) if int(p) == p else int(p) + 1)
            sys.stdout.write(str(self.back) + left + str(Style.reset_all))
            for v in self.data:
                v.draw()
            sys.stdout.write(str(self.back) + right + str(Style.reset_all))
            self.real_height = 1
        elif self.visibility == Visibility.invisible:
            sys.stdout.write(' ' * self.real_width)
            self.real_height = 1

    def clear(self):
        sys.stdout.write(Cursor.UP(self.real_height) + clear_line())

    def update_width(self, parent_width):
        """

        :param parent_width:
        :return:
        :rtype:int
        """

        if self.width == Width.fill:
            self.real_width = parent_width
        elif self.width == Width.wrap:
            pass
        else:
            self.real_width = self.width

        weight_view = []
        all_weight = 0

        if self.width == Width.wrap:
            _width = parent_width
        else:
            _width = self.real_width

        for v in self.data:
            if v.weight:
                weight_view.append(v)
                all_weight += v.weight
                continue
            child_width = v.update_width(_width)
            _width -= child_width
            _width = max(0, _width)

        if weight_view:
            per_width = _width / float(all_weight)
            for v in weight_view:
                v.real_width = int(v.weight * per_width)

        self.child_width = sum([v.real_width for v in self.data])

        if self.width == Width.wrap:
            self.real_width = self.child_width

        return self.real_width


class TableLayout(View):

    def __init__(self, id, width=Width.fill, height=1, visibility=Visibility.visible):
        """

        :param id:
        :param width:
        :param height: no used

        :type id:str
        :type width:
        :type height:int
        :type visibility:str
        """

        super(TableLayout, self).__init__(id, width, height, visibility, Gravity.left)
        self.data = []  # type: list[TableRow]

    @classmethod
    def quick_init(cls, id, data, width=Width.fill, height=1, visibility=Visibility.visible):
        """

        :param id:
        :param data:
        :param width:
        :param height:
        :param visibility:
        :return:

        :rtype:TableLayout
        """

        table = cls(id, width, height, visibility)
        table.data = data
        return table

    def add_view(self, v):

        self.data.append(v)

    def add_views(self, *views):

        self.data += views

    def add_view_list(self, views):
        self.data += views

    def update_width(self, parent_width):
        """

        :param parent_width:
        :return:
        :rtype: int
        """

        if self.width == Width.fill:
            self.real_width = parent_width
        elif self.width == Width.wrap:
            self.real_width = 0
        else:
            self.real_width = self.width

        for row in self.data:
            if self.width == Width.wrap:
                child_width = row.update_width(parent_width)
                self.real_width = max(self.real_width, child_width)
            else:
                row.update_width(self.real_width)

        return self.real_width

    def draw(self):
        self.real_height = 0
        is_first = True
        for r in self.data:
            if r.visibility != Visibility.gone:
                if not is_first:
                    sys.stdout.write('\n')
                else:
                    is_first = False
                r.draw()
                self.real_height += 1

    def clear(self):
        while self.real_height:
            sys.stdout.write(Cursor.UP(1) + clear_line())
            self.real_height -= 1


class LinearLayout(View):

    def __init__(self, id, width, height=1, visibility=Visibility.visible, gravity=Gravity.left,
                 orientation=Orientation.vertical):
        """

        :param id:
        :param width:
        :param height: no used

        :type id:str
        :type width:
        :type height:int
        :type visibility:str
        :type gravity:str
        """

        super(LinearLayout, self).__init__(id, width, height, visibility, gravity)
        if orientation == Orientation.vertical:
            self.end_code = '\n'
        elif orientation == Orientation.horizon:
            self.end_code = ''



# -*- coding: utf-8 -*-

from terminal_layout.view.params import Visibility, Gravity, Width
from terminal_layout.view.layout import TableLayout, TableRow
from terminal_layout.view.text_view import TextView




# -*- coding: utf-8 -*-
# Initially taken from:
# http://code.activestate.com/recipes/134892/
# Thanks to Danny Yoo
import os
import sys
import tty
import termios

from terminal_layout.readkey.key import Key


def readkey():
    fd = sys.stdin.fileno()
    old_settings = termios.tcgetattr(fd)
    try:
        tty.setraw(fd)
        c = os.read(fd, 4)
        if isinstance(c, str):
            # py2
            c = c.decode('utf-8')
        else:
            # py3
            try:
                c = str(c, encoding='utf-8')
            except:
                c = ''
    finally:
        termios.tcsetattr(fd, termios.TCSADRAIN, old_settings)
    if c == Key.CTRL_C:
        raise KeyboardInterrupt
    return c


def readkey2():
    fd = sys.stdin.fileno()
    old_settings = termios.tcgetattr(fd)
    # 配置终端
    new_settings = old_settings[:]

    # 使用非规范模式(索引3是c_lflag 也就是本地模式)
    new_settings[3] &= ~termios.ICANON
    # # 关闭回显(输入不会被显示)
    new_settings[3] &= ~termios.ECHO
    try:
        termios.tcsetattr(fd, termios.TCSANOW, new_settings)
        c = os.read(fd, 4)
        if isinstance(c, str):
            # py2
            c = c.decode('utf-8')
        else:
            # py3
            try:
                c = str(c, encoding='utf-8')
            except:
                c = ''
    finally:
        termios.tcsetattr(fd, termios.TCSADRAIN, old_settings)
    if c == Key.CTRL_C:
        raise KeyboardInterrupt
    return c



# -*- coding: utf-8 -*-
# -*- coding: utf-8 -*-
from terminal_layout import *

print('\nstyle support window, linux, osx\n')

ctl = LayoutCtl.quick(TableLayout,
                      [
                          [TextView('', 'style = bright', style=Style.bright)],
                          [TextView('', 'style = dim', style=Style.dim)],
                          [TextView('', 'style = normal', style=Style.normal)]
                      ])

ctl.draw(auto_re_draw=False)




from terminal_layout import *
from terminal_layout.helper.class_helper import instance_variables


class StringStyle(object):

    def __init__(self, fore=None, back=None, style=None):
        self.fore = fore or ''
        self.back = back or ''
        self.style = style or ''

    def to_dict(self):
        return {
            'fore': self.fore,
            'back': self.back,
            'style': self.style
        }


class Choice(object):
    @instance_variables
    def __init__(self, title, choices, icon='> ', icon_style=StringStyle(fore=Fore.green), choices_style=StringStyle(),
                 selected_style=StringStyle(), loop=True):
        self.current = 0
        self.result = None

    def get_choice(self):
        views = [[TextView('', self.title)]]
        for i, c in enumerate(self.choices):
            views.append(
                [TextView('icon%d' % i, self.icon, visibility=Visibility.invisible, **self.icon_style.to_dict()),
                 TextView('value%d' % i, c, **self.choices_style.to_dict())])

        views[1][0].visibility = Visibility.visible
        views[1][1] = TextView('value0', self.choices[0], **self.selected_style.to_dict())
        self.ctl = LayoutCtl.quick(TableLayout,
                                   views)
        self.ctl.draw(auto_re_draw=False)

        kl = KeyListener()
        kl.bind_key(Key.UP, Key.DOWN, self.change_current, decorator=False)
        kl.bind_key(Key.ENTER, self.select, decorator=False)

        kl.listen()
        return self.result

    def update_style(self, view, style):
        view.set_back(style.back)
        view.set_style(style.style)
        view.set_fore(style.fore)

    def change_current(self, kl, event):
        temp = self.current
        if event.key == Key.UP:
            temp -= 1
        elif event.key == Key.DOWN:
            temp += 1
        if temp < 0:
            temp = len(self.choices) - 1 if self.loop else 0
        if temp >= len(self.choices):
            temp = 0 if self.loop else len(self.choices) - 1
        self.ctl.find_view_by_id('icon%d' % self.current).set_visibility(Visibility.invisible)
        self.update_style(self.ctl.find_view_by_id('value%d' % self.current), self.choices_style)

        self.current = temp
        self.ctl.find_view_by_id('icon%d' % self.current).set_visibility(Visibility.visible)
        self.update_style(self.ctl.find_view_by_id('value%d' % self.current), self.selected_style)

        self.ctl.re_draw()

    def select(self, kl, event):
        self.result = (self.current, self.choices[self.current])
        kl.stop()



# -*- coding: utf-8 -*-

char_to_length = {
    '\t': 4,
}
char_to_str = {
    '\t': ' ' * 4,
    '\n': ' ',
}


def add_char_length_map(d):
    """
    ex:
    add_char_length_map({'a':1})
    :param d: 
    :type d: dict
    """
    char_to_length.update(d)


class Char(object):
    length = None

    def __init__(self, c):
        if len(c) != 1:
            raise TypeError('expected a character, but string of length %d found' % len(c))
        self.c = c

    def __len__(self):
        if self.length is not None:
            return self.length

        temp = char_to_length.get(self.c, None)
        if temp:
            self.length = temp
            return self.length

        if ord(self.c) < 11904:
            self.length = 1
        else:
            self.length = 2
        return self.length

    def __str__(self):
        return char_to_str.get(self.c, self.c)


class String(object):
    """

    ex:

    >>> s=String('a啊啊')
    >>> len(s)
    5
    >>> s[:2]
    'a'
    >>> s[:3]
    'a啊'

    """
    length = None
    char_list = None

    def __init__(self, text):
        """

        :param text:
        :type text:str
        """
        self.char_list = []  # type: list[Char]
        self.origin_text = text
        for c in text:
            self.char_list.append(Char(c))

    def __len__(self):
        if self.length is not None:
            return self.length

        t = 0
        for c in self.char_list:
            t += len(c)
        self.length = t

        return self.length

    def __str__(self):
        return ''.join(str(c) for c in self.char_list)

    def __getitem__(self, item):
        """
        :rtype: str
        """""
        start = item.start or 0
        if start != 0:
            raise TypeError('slice start must be 0 or None')
        stop = item.stop
        if stop is None or stop < 0:
            raise TypeError('slice stop must be positive integer')

        s = ''
        for c in self.char_list:
            length = len(c)
            if stop >= length:
                s += str(c)
            else:
                break
            stop -= length

        return s



# -*- coding: utf-8 -*-
# -*- coding: utf-8 -*-
from terminal_layout import *

ctl = LayoutCtl.quick(TableLayout,
                      [
                          [TextView('', 'weight = 1 ', weight=1, back=Back.cyan),
                           TextView('', 'weight = 1 ', weight=1, back=Back.green)],
                          [TextView('', 'weight = 3 ', weight=3, back=Back.cyan),
                           TextView('', 'weight = 1 ', weight=1, back=Back.green)],
                          [TextView('', 'weight = 1 ', weight=1, back=Back.cyan),
                           TextView('', 'width = 15 ', width=15, back=Back.green)],
                          [TextView('', 'weight = 2 ', weight=1, back=Back.cyan),
                           TextView('', 'width = 15 ', width=15, back=Back.green)],
                          [TextView('', 'weight = 1 ', weight=1, back=Back.cyan),
                           TextView('', 'width = wrap ', width=Width.wrap, back=Back.green)],

                      ])

t = ctl.get_layout()
t.set_width(40)

ctl.draw(auto_re_draw=False)



# -*- coding: utf-8 -*-
from terminal_layout import *

ctl = LayoutCtl.quick(TableLayout,
                      [
                          [TextView('', 'width = 12', width=12, back=Back.green)],
                          [TextView('', 'width = warp', width=Width.wrap, back=Back.green)],
                          [TextView('', 'width = fill', width=Width.fill, back=Back.green)],
                      ])

ctl.get_layout().set_width(50)

ctl.draw(auto_re_draw=False)



# -*- coding: utf-8 -*-
from terminal_layout import *



print('\n[ex]color support linux, osx\n')

ctl = LayoutCtl.quick(TableLayout,
                      [
                          [TextView('', 'fore = ex_black', width=30, fore=Fore.ex_black),
                           TextView('', 'back = ex_black', back=Back.ex_black)],
                          [TextView('', 'fore = ex_red', width=30, fore=Fore.ex_red),
                           TextView('', 'back = ex_red', back=Back.ex_red)],
                          [TextView('', 'fore = ex_green', width=30, fore=Fore.ex_green),
                           TextView('', 'back = ex_green', back=Back.ex_green)],
                          [TextView('', 'fore = ex_yellow', width=30, fore=Fore.ex_yellow),
                           TextView('', 'back = ex_yellow', back=Back.ex_yellow)],
                          [TextView('', 'fore = ex_blue', width=30, fore=Fore.ex_blue),
                           TextView('', 'back = ex_blue', back=Back.ex_blue)],
                          [TextView('', 'fore = ex_magenta', width=30, fore=Fore.ex_magenta),
                           TextView('', 'back = ex_magenta', back=Back.ex_magenta)],
                          [TextView('', 'fore = ex_cyan', width=30, fore=Fore.ex_cyan),
                           TextView('', 'back = ex_cyan', back=Back.ex_cyan)],
                          [TextView('', 'fore = ex_light_gray', width=30, fore=Fore.ex_light_gray),
                           TextView('', 'back = ex_light_gray', back=Back.ex_light_gray)],
                          [TextView('', 'fore = ex_dark_gray', width=30, fore=Fore.ex_dark_gray),
                           TextView('', 'back = ex_dark_gray', back=Back.ex_dark_gray)],
                          [TextView('', 'fore = ex_light_red', width=30, fore=Fore.ex_light_red),
                           TextView('', 'back = ex_light_red', back=Back.ex_light_red)],
                          [TextView('', 'fore = ex_light_green', width=30, fore=Fore.ex_light_green),
                           TextView('', 'back = ex_light_green', back=Back.ex_light_green)],
                          [TextView('', 'fore = ex_light_yellow', width=30, fore=Fore.ex_light_yellow),
                           TextView('', 'back = ex_light_yellow', back=Back.ex_light_yellow)],
                          [TextView('', 'fore = ex_light_blue', width=30, fore=Fore.ex_light_blue),
                           TextView('', 'back = ex_light_blue', back=Back.ex_light_blue)],
                          [TextView('', 'fore = ex_light_magenta', width=30, fore=Fore.ex_light_magenta),
                           TextView('', 'back = ex_light_magenta', back=Back.ex_light_magenta)],
                          [TextView('', 'fore = ex_light_cyan', width=30, fore=Fore.ex_light_cyan),
                           TextView('', 'back = ex_light_cyan', back=Back.ex_light_cyan)],
                          [TextView('', 'fore = ex_white', width=30, fore=Fore.ex_white),
                           TextView('', 'back = ex_white', back=Back.ex_white)],
                          [TextView('', 'fore = ex_grey_0', width=30, fore=Fore.ex_grey_0),
                           TextView('', 'back = ex_grey_0', back=Back.ex_grey_0)],
                          [TextView('', 'fore = ex_navy_blue', width=30, fore=Fore.ex_navy_blue),
                           TextView('', 'back = ex_navy_blue', back=Back.ex_navy_blue)],
                          [TextView('', 'fore = ex_dark_blue', width=30, fore=Fore.ex_dark_blue),
                           TextView('', 'back = ex_dark_blue', back=Back.ex_dark_blue)],
                          [TextView('', 'fore = ex_blue_3a', width=30, fore=Fore.ex_blue_3a),
                           TextView('', 'back = ex_blue_3a', back=Back.ex_blue_3a)],
                          [TextView('', 'fore = ex_blue_3b', width=30, fore=Fore.ex_blue_3b),
                           TextView('', 'back = ex_blue_3b', back=Back.ex_blue_3b)],
                          [TextView('', 'fore = ex_blue_1', width=30, fore=Fore.ex_blue_1),
                           TextView('', 'back = ex_blue_1', back=Back.ex_blue_1)],
                          [TextView('', 'fore = ex_dark_green', width=30, fore=Fore.ex_dark_green),
                           TextView('', 'back = ex_dark_green', back=Back.ex_dark_green)],
                          [TextView('', 'fore = ex_deep_sky_blue_4a', width=30, fore=Fore.ex_deep_sky_blue_4a),
                           TextView('', 'back = ex_deep_sky_blue_4a', back=Back.ex_deep_sky_blue_4a)],
                          [TextView('', 'fore = ex_deep_sky_blue_4b', width=30, fore=Fore.ex_deep_sky_blue_4b),
                           TextView('', 'back = ex_deep_sky_blue_4b', back=Back.ex_deep_sky_blue_4b)],
                          [TextView('', 'fore = ex_deep_sky_blue_4c', width=30, fore=Fore.ex_deep_sky_blue_4c),
                           TextView('', 'back = ex_deep_sky_blue_4c', back=Back.ex_deep_sky_blue_4c)],
                          [TextView('', 'fore = ex_dodger_blue_3', width=30, fore=Fore.ex_dodger_blue_3),
                           TextView('', 'back = ex_dodger_blue_3', back=Back.ex_dodger_blue_3)],
                          [TextView('', 'fore = ex_dodger_blue_2', width=30, fore=Fore.ex_dodger_blue_2),
                           TextView('', 'back = ex_dodger_blue_2', back=Back.ex_dodger_blue_2)],
                          [TextView('', 'fore = ex_green_4', width=30, fore=Fore.ex_green_4),
                           TextView('', 'back = ex_green_4', back=Back.ex_green_4)],
                          [TextView('', 'fore = ex_spring_green_4', width=30, fore=Fore.ex_spring_green_4),
                           TextView('', 'back = ex_spring_green_4', back=Back.ex_spring_green_4)],
                          [TextView('', 'fore = ex_turquoise_4', width=30, fore=Fore.ex_turquoise_4),
                           TextView('', 'back = ex_turquoise_4', back=Back.ex_turquoise_4)],
                          [TextView('', 'fore = ex_deep_sky_blue_3a', width=30, fore=Fore.ex_deep_sky_blue_3a),
                           TextView('', 'back = ex_deep_sky_blue_3a', back=Back.ex_deep_sky_blue_3a)],
                          [TextView('', 'fore = ex_deep_sky_blue_3b', width=30, fore=Fore.ex_deep_sky_blue_3b),
                           TextView('', 'back = ex_deep_sky_blue_3b', back=Back.ex_deep_sky_blue_3b)],
                          [TextView('', 'fore = ex_dodger_blue_1', width=30, fore=Fore.ex_dodger_blue_1),
                           TextView('', 'back = ex_dodger_blue_1', back=Back.ex_dodger_blue_1)],
                          [TextView('', 'fore = ex_green_3a', width=30, fore=Fore.ex_green_3a),
                           TextView('', 'back = ex_green_3a', back=Back.ex_green_3a)],
                          [TextView('', 'fore = ex_spring_green_3a', width=30, fore=Fore.ex_spring_green_3a),
                           TextView('', 'back = ex_spring_green_3a', back=Back.ex_spring_green_3a)],
                          [TextView('', 'fore = ex_dark_cyan', width=30, fore=Fore.ex_dark_cyan),
                           TextView('', 'back = ex_dark_cyan', back=Back.ex_dark_cyan)],
                          [TextView('', 'fore = ex_light_sea_green', width=30, fore=Fore.ex_light_sea_green),
                           TextView('', 'back = ex_light_sea_green', back=Back.ex_light_sea_green)],
                          [TextView('', 'fore = ex_deep_sky_blue_2', width=30, fore=Fore.ex_deep_sky_blue_2),
                           TextView('', 'back = ex_deep_sky_blue_2', back=Back.ex_deep_sky_blue_2)],
                          [TextView('', 'fore = ex_deep_sky_blue_1', width=30, fore=Fore.ex_deep_sky_blue_1),
                           TextView('', 'back = ex_deep_sky_blue_1', back=Back.ex_deep_sky_blue_1)],
                          [TextView('', 'fore = ex_green_3b', width=30, fore=Fore.ex_green_3b),
                           TextView('', 'back = ex_green_3b', back=Back.ex_green_3b)],
                          [TextView('', 'fore = ex_spring_green_3b', width=30, fore=Fore.ex_spring_green_3b),
                           TextView('', 'back = ex_spring_green_3b', back=Back.ex_spring_green_3b)],
                          [TextView('', 'fore = ex_spring_green_2a', width=30, fore=Fore.ex_spring_green_2a),
                           TextView('', 'back = ex_spring_green_2a', back=Back.ex_spring_green_2a)],
                          [TextView('', 'fore = ex_cyan_3', width=30, fore=Fore.ex_cyan_3),
                           TextView('', 'back = ex_cyan_3', back=Back.ex_cyan_3)],
                          [TextView('', 'fore = ex_dark_turquoise', width=30, fore=Fore.ex_dark_turquoise),
                           TextView('', 'back = ex_dark_turquoise', back=Back.ex_dark_turquoise)],
                          [TextView('', 'fore = ex_turquoise_2', width=30, fore=Fore.ex_turquoise_2),
                           TextView('', 'back = ex_turquoise_2', back=Back.ex_turquoise_2)],
                          [TextView('', 'fore = ex_green_1', width=30, fore=Fore.ex_green_1),
                           TextView('', 'back = ex_green_1', back=Back.ex_green_1)],
                          [TextView('', 'fore = ex_spring_green_2b', width=30, fore=Fore.ex_spring_green_2b),
                           TextView('', 'back = ex_spring_green_2b', back=Back.ex_spring_green_2b)],
                          [TextView('', 'fore = ex_spring_green_1', width=30, fore=Fore.ex_spring_green_1),
                           TextView('', 'back = ex_spring_green_1', back=Back.ex_spring_green_1)],
                          [TextView('', 'fore = ex_medium_spring_green', width=30, fore=Fore.ex_medium_spring_green),
                           TextView('', 'back = ex_medium_spring_green', back=Back.ex_medium_spring_green)],
                          [TextView('', 'fore = ex_cyan_2', width=30, fore=Fore.ex_cyan_2),
                           TextView('', 'back = ex_cyan_2', back=Back.ex_cyan_2)],
                          [TextView('', 'fore = ex_cyan_1', width=30, fore=Fore.ex_cyan_1),
                           TextView('', 'back = ex_cyan_1', back=Back.ex_cyan_1)],
                          [TextView('', 'fore = ex_dark_red_1', width=30, fore=Fore.ex_dark_red_1),
                           TextView('', 'back = ex_dark_red_1', back=Back.ex_dark_red_1)],
                          [TextView('', 'fore = ex_deep_pink_4a', width=30, fore=Fore.ex_deep_pink_4a),
                           TextView('', 'back = ex_deep_pink_4a', back=Back.ex_deep_pink_4a)],
                          [TextView('', 'fore = ex_purple_4a', width=30, fore=Fore.ex_purple_4a),
                           TextView('', 'back = ex_purple_4a', back=Back.ex_purple_4a)],
                          [TextView('', 'fore = ex_purple_4b', width=30, fore=Fore.ex_purple_4b),
                           TextView('', 'back = ex_purple_4b', back=Back.ex_purple_4b)],
                          [TextView('', 'fore = ex_purple_3', width=30, fore=Fore.ex_purple_3),
                           TextView('', 'back = ex_purple_3', back=Back.ex_purple_3)],
                          [TextView('', 'fore = ex_blue_violet', width=30, fore=Fore.ex_blue_violet),
                           TextView('', 'back = ex_blue_violet', back=Back.ex_blue_violet)],
                          [TextView('', 'fore = ex_orange_4a', width=30, fore=Fore.ex_orange_4a),
                           TextView('', 'back = ex_orange_4a', back=Back.ex_orange_4a)],
                          [TextView('', 'fore = ex_grey_37', width=30, fore=Fore.ex_grey_37),
                           TextView('', 'back = ex_grey_37', back=Back.ex_grey_37)],
                          [TextView('', 'fore = ex_medium_purple_4', width=30, fore=Fore.ex_medium_purple_4),
                           TextView('', 'back = ex_medium_purple_4', back=Back.ex_medium_purple_4)],
                          [TextView('', 'fore = ex_slate_blue_3a', width=30, fore=Fore.ex_slate_blue_3a),
                           TextView('', 'back = ex_slate_blue_3a', back=Back.ex_slate_blue_3a)],
                          [TextView('', 'fore = ex_slate_blue_3b', width=30, fore=Fore.ex_slate_blue_3b),
                           TextView('', 'back = ex_slate_blue_3b', back=Back.ex_slate_blue_3b)],
                          [TextView('', 'fore = ex_royal_blue_1', width=30, fore=Fore.ex_royal_blue_1),
                           TextView('', 'back = ex_royal_blue_1', back=Back.ex_royal_blue_1)],
                          [TextView('', 'fore = ex_chartreuse_4', width=30, fore=Fore.ex_chartreuse_4),
                           TextView('', 'back = ex_chartreuse_4', back=Back.ex_chartreuse_4)],
                          [TextView('', 'fore = ex_dark_sea_green_4a', width=30, fore=Fore.ex_dark_sea_green_4a),
                           TextView('', 'back = ex_dark_sea_green_4a', back=Back.ex_dark_sea_green_4a)],
                          [TextView('', 'fore = ex_pale_turquoise_4', width=30, fore=Fore.ex_pale_turquoise_4),
                           TextView('', 'back = ex_pale_turquoise_4', back=Back.ex_pale_turquoise_4)],
                          [TextView('', 'fore = ex_steel_blue', width=30, fore=Fore.ex_steel_blue),
                           TextView('', 'back = ex_steel_blue', back=Back.ex_steel_blue)],
                          [TextView('', 'fore = ex_steel_blue_3', width=30, fore=Fore.ex_steel_blue_3),
                           TextView('', 'back = ex_steel_blue_3', back=Back.ex_steel_blue_3)],
                          [TextView('', 'fore = ex_cornflower_blue', width=30, fore=Fore.ex_cornflower_blue),
                           TextView('', 'back = ex_cornflower_blue', back=Back.ex_cornflower_blue)],
                          [TextView('', 'fore = ex_chartreuse_3a', width=30, fore=Fore.ex_chartreuse_3a),
                           TextView('', 'back = ex_chartreuse_3a', back=Back.ex_chartreuse_3a)],
                          [TextView('', 'fore = ex_dark_sea_green_4b', width=30, fore=Fore.ex_dark_sea_green_4b),
                           TextView('', 'back = ex_dark_sea_green_4b', back=Back.ex_dark_sea_green_4b)],
                          [TextView('', 'fore = ex_cadet_blue_2', width=30, fore=Fore.ex_cadet_blue_2),
                           TextView('', 'back = ex_cadet_blue_2', back=Back.ex_cadet_blue_2)],
                          [TextView('', 'fore = ex_cadet_blue_1', width=30, fore=Fore.ex_cadet_blue_1),
                           TextView('', 'back = ex_cadet_blue_1', back=Back.ex_cadet_blue_1)],
                          [TextView('', 'fore = ex_sky_blue_3', width=30, fore=Fore.ex_sky_blue_3),
                           TextView('', 'back = ex_sky_blue_3', back=Back.ex_sky_blue_3)],
                          [TextView('', 'fore = ex_steel_blue_1a', width=30, fore=Fore.ex_steel_blue_1a),
                           TextView('', 'back = ex_steel_blue_1a', back=Back.ex_steel_blue_1a)],
                          [TextView('', 'fore = ex_chartreuse_3b', width=30, fore=Fore.ex_chartreuse_3b),
                           TextView('', 'back = ex_chartreuse_3b', back=Back.ex_chartreuse_3b)],
                          [TextView('', 'fore = ex_pale_green_3a', width=30, fore=Fore.ex_pale_green_3a),
                           TextView('', 'back = ex_pale_green_3a', back=Back.ex_pale_green_3a)],
                          [TextView('', 'fore = ex_sea_green_3', width=30, fore=Fore.ex_sea_green_3),
                           TextView('', 'back = ex_sea_green_3', back=Back.ex_sea_green_3)],
                          [TextView('', 'fore = ex_aquamarine_3', width=30, fore=Fore.ex_aquamarine_3),
                           TextView('', 'back = ex_aquamarine_3', back=Back.ex_aquamarine_3)],
                          [TextView('', 'fore = ex_medium_turquoise', width=30, fore=Fore.ex_medium_turquoise),
                           TextView('', 'back = ex_medium_turquoise', back=Back.ex_medium_turquoise)],
                          [TextView('', 'fore = ex_steel_blue_1b', width=30, fore=Fore.ex_steel_blue_1b),
                           TextView('', 'back = ex_steel_blue_1b', back=Back.ex_steel_blue_1b)],
                          [TextView('', 'fore = ex_chartreuse_2a', width=30, fore=Fore.ex_chartreuse_2a),
                           TextView('', 'back = ex_chartreuse_2a', back=Back.ex_chartreuse_2a)],
                          [TextView('', 'fore = ex_sea_green_2', width=30, fore=Fore.ex_sea_green_2),
                           TextView('', 'back = ex_sea_green_2', back=Back.ex_sea_green_2)],
                          [TextView('', 'fore = ex_sea_green_1a', width=30, fore=Fore.ex_sea_green_1a),
                           TextView('', 'back = ex_sea_green_1a', back=Back.ex_sea_green_1a)],
                          [TextView('', 'fore = ex_sea_green_1b', width=30, fore=Fore.ex_sea_green_1b),
                           TextView('', 'back = ex_sea_green_1b', back=Back.ex_sea_green_1b)],
                          [TextView('', 'fore = ex_aquamarine_1a', width=30, fore=Fore.ex_aquamarine_1a),
                           TextView('', 'back = ex_aquamarine_1a', back=Back.ex_aquamarine_1a)],
                          [TextView('', 'fore = ex_dark_slate_gray_2', width=30, fore=Fore.ex_dark_slate_gray_2),
                           TextView('', 'back = ex_dark_slate_gray_2', back=Back.ex_dark_slate_gray_2)],
                          [TextView('', 'fore = ex_dark_red_2', width=30, fore=Fore.ex_dark_red_2),
                           TextView('', 'back = ex_dark_red_2', back=Back.ex_dark_red_2)],
                          [TextView('', 'fore = ex_deep_pink_4b', width=30, fore=Fore.ex_deep_pink_4b),
                           TextView('', 'back = ex_deep_pink_4b', back=Back.ex_deep_pink_4b)],
                          [TextView('', 'fore = ex_dark_magenta_1', width=30, fore=Fore.ex_dark_magenta_1),
                           TextView('', 'back = ex_dark_magenta_1', back=Back.ex_dark_magenta_1)],
                          [TextView('', 'fore = ex_dark_magenta_2', width=30, fore=Fore.ex_dark_magenta_2),
                           TextView('', 'back = ex_dark_magenta_2', back=Back.ex_dark_magenta_2)],
                          [TextView('', 'fore = ex_dark_violet_1a', width=30, fore=Fore.ex_dark_violet_1a),
                           TextView('', 'back = ex_dark_violet_1a', back=Back.ex_dark_violet_1a)],
                          [TextView('', 'fore = ex_purple_1a', width=30, fore=Fore.ex_purple_1a),
                           TextView('', 'back = ex_purple_1a', back=Back.ex_purple_1a)],
                          [TextView('', 'fore = ex_orange_4b', width=30, fore=Fore.ex_orange_4b),
                           TextView('', 'back = ex_orange_4b', back=Back.ex_orange_4b)],
                          [TextView('', 'fore = ex_light_pink_4', width=30, fore=Fore.ex_light_pink_4),
                           TextView('', 'back = ex_light_pink_4', back=Back.ex_light_pink_4)],
                          [TextView('', 'fore = ex_plum_4', width=30, fore=Fore.ex_plum_4),
                           TextView('', 'back = ex_plum_4', back=Back.ex_plum_4)],
                          [TextView('', 'fore = ex_medium_purple_3a', width=30, fore=Fore.ex_medium_purple_3a),
                           TextView('', 'back = ex_medium_purple_3a', back=Back.ex_medium_purple_3a)],
                          [TextView('', 'fore = ex_medium_purple_3b', width=30, fore=Fore.ex_medium_purple_3b),
                           TextView('', 'back = ex_medium_purple_3b', back=Back.ex_medium_purple_3b)],
                          [TextView('', 'fore = ex_slate_blue_1', width=30, fore=Fore.ex_slate_blue_1),
                           TextView('', 'back = ex_slate_blue_1', back=Back.ex_slate_blue_1)],
                          [TextView('', 'fore = ex_yellow_4a', width=30, fore=Fore.ex_yellow_4a),
                           TextView('', 'back = ex_yellow_4a', back=Back.ex_yellow_4a)],
                          [TextView('', 'fore = ex_wheat_4', width=30, fore=Fore.ex_wheat_4),
                           TextView('', 'back = ex_wheat_4', back=Back.ex_wheat_4)],
                          [TextView('', 'fore = ex_grey_53', width=30, fore=Fore.ex_grey_53),
                           TextView('', 'back = ex_grey_53', back=Back.ex_grey_53)],
                          [TextView('', 'fore = ex_light_slate_grey', width=30, fore=Fore.ex_light_slate_grey),
                           TextView('', 'back = ex_light_slate_grey', back=Back.ex_light_slate_grey)],
                          [TextView('', 'fore = ex_medium_purple', width=30, fore=Fore.ex_medium_purple),
                           TextView('', 'back = ex_medium_purple', back=Back.ex_medium_purple)],
                          [TextView('', 'fore = ex_light_slate_blue', width=30, fore=Fore.ex_light_slate_blue),
                           TextView('', 'back = ex_light_slate_blue', back=Back.ex_light_slate_blue)],
                          [TextView('', 'fore = ex_yellow_4b', width=30, fore=Fore.ex_yellow_4b),
                           TextView('', 'back = ex_yellow_4b', back=Back.ex_yellow_4b)],
                          [TextView('', 'fore = ex_dark_olive_green_3a', width=30, fore=Fore.ex_dark_olive_green_3a),
                           TextView('', 'back = ex_dark_olive_green_3a', back=Back.ex_dark_olive_green_3a)],
                          [TextView('', 'fore = ex_dark_green_sea', width=30, fore=Fore.ex_dark_green_sea),
                           TextView('', 'back = ex_dark_green_sea', back=Back.ex_dark_green_sea)],
                          [TextView('', 'fore = ex_light_sky_blue_3a', width=30, fore=Fore.ex_light_sky_blue_3a),
                           TextView('', 'back = ex_light_sky_blue_3a', back=Back.ex_light_sky_blue_3a)],
                          [TextView('', 'fore = ex_light_sky_blue_3b', width=30, fore=Fore.ex_light_sky_blue_3b),
                           TextView('', 'back = ex_light_sky_blue_3b', back=Back.ex_light_sky_blue_3b)],
                          [TextView('', 'fore = ex_sky_blue_2', width=30, fore=Fore.ex_sky_blue_2),
                           TextView('', 'back = ex_sky_blue_2', back=Back.ex_sky_blue_2)],
                          [TextView('', 'fore = ex_chartreuse_2b', width=30, fore=Fore.ex_chartreuse_2b),
                           TextView('', 'back = ex_chartreuse_2b', back=Back.ex_chartreuse_2b)],
                          [TextView('', 'fore = ex_dark_olive_green_3b', width=30, fore=Fore.ex_dark_olive_green_3b),
                           TextView('', 'back = ex_dark_olive_green_3b', back=Back.ex_dark_olive_green_3b)],
                          [TextView('', 'fore = ex_pale_green_3b', width=30, fore=Fore.ex_pale_green_3b),
                           TextView('', 'back = ex_pale_green_3b', back=Back.ex_pale_green_3b)],
                          [TextView('', 'fore = ex_dark_sea_green_3a', width=30, fore=Fore.ex_dark_sea_green_3a),
                           TextView('', 'back = ex_dark_sea_green_3a', back=Back.ex_dark_sea_green_3a)],
                          [TextView('', 'fore = ex_dark_slate_gray_3', width=30, fore=Fore.ex_dark_slate_gray_3),
                           TextView('', 'back = ex_dark_slate_gray_3', back=Back.ex_dark_slate_gray_3)],
                          [TextView('', 'fore = ex_sky_blue_1', width=30, fore=Fore.ex_sky_blue_1),
                           TextView('', 'back = ex_sky_blue_1', back=Back.ex_sky_blue_1)],
                          [TextView('', 'fore = ex_chartreuse_1', width=30, fore=Fore.ex_chartreuse_1),
                           TextView('', 'back = ex_chartreuse_1', back=Back.ex_chartreuse_1)],
                          [TextView('', 'fore = ex_light_green_2', width=30, fore=Fore.ex_light_green_2),
                           TextView('', 'back = ex_light_green_2', back=Back.ex_light_green_2)],
                          [TextView('', 'fore = ex_light_green_3', width=30, fore=Fore.ex_light_green_3),
                           TextView('', 'back = ex_light_green_3', back=Back.ex_light_green_3)],
                          [TextView('', 'fore = ex_pale_green_1a', width=30, fore=Fore.ex_pale_green_1a),
                           TextView('', 'back = ex_pale_green_1a', back=Back.ex_pale_green_1a)],
                          [TextView('', 'fore = ex_aquamarine_1b', width=30, fore=Fore.ex_aquamarine_1b),
                           TextView('', 'back = ex_aquamarine_1b', back=Back.ex_aquamarine_1b)],
                          [TextView('', 'fore = ex_dark_slate_gray_1', width=30, fore=Fore.ex_dark_slate_gray_1),
                           TextView('', 'back = ex_dark_slate_gray_1', back=Back.ex_dark_slate_gray_1)],
                          [TextView('', 'fore = ex_red_3a', width=30, fore=Fore.ex_red_3a),
                           TextView('', 'back = ex_red_3a', back=Back.ex_red_3a)],
                          [TextView('', 'fore = ex_deep_pink_4c', width=30, fore=Fore.ex_deep_pink_4c),
                           TextView('', 'back = ex_deep_pink_4c', back=Back.ex_deep_pink_4c)],
                          [TextView('', 'fore = ex_medium_violet_red', width=30, fore=Fore.ex_medium_violet_red),
                           TextView('', 'back = ex_medium_violet_red', back=Back.ex_medium_violet_red)],
                          [TextView('', 'fore = ex_magenta_3a', width=30, fore=Fore.ex_magenta_3a),
                           TextView('', 'back = ex_magenta_3a', back=Back.ex_magenta_3a)],
                          [TextView('', 'fore = ex_dark_violet_1b', width=30, fore=Fore.ex_dark_violet_1b),
                           TextView('', 'back = ex_dark_violet_1b', back=Back.ex_dark_violet_1b)],
                          [TextView('', 'fore = ex_purple_1b', width=30, fore=Fore.ex_purple_1b),
                           TextView('', 'back = ex_purple_1b', back=Back.ex_purple_1b)],
                          [TextView('', 'fore = ex_dark_orange_3a', width=30, fore=Fore.ex_dark_orange_3a),
                           TextView('', 'back = ex_dark_orange_3a', back=Back.ex_dark_orange_3a)],
                          [TextView('', 'fore = ex_indian_red_1a', width=30, fore=Fore.ex_indian_red_1a),
                           TextView('', 'back = ex_indian_red_1a', back=Back.ex_indian_red_1a)],
                          [TextView('', 'fore = ex_hot_pink_3a', width=30, fore=Fore.ex_hot_pink_3a),
                           TextView('', 'back = ex_hot_pink_3a', back=Back.ex_hot_pink_3a)],
                          [TextView('', 'fore = ex_medium_orchid_3', width=30, fore=Fore.ex_medium_orchid_3),
                           TextView('', 'back = ex_medium_orchid_3', back=Back.ex_medium_orchid_3)],
                          [TextView('', 'fore = ex_medium_orchid', width=30, fore=Fore.ex_medium_orchid),
                           TextView('', 'back = ex_medium_orchid', back=Back.ex_medium_orchid)],
                          [TextView('', 'fore = ex_medium_purple_2a', width=30, fore=Fore.ex_medium_purple_2a),
                           TextView('', 'back = ex_medium_purple_2a', back=Back.ex_medium_purple_2a)],
                          [TextView('', 'fore = ex_dark_goldenrod', width=30, fore=Fore.ex_dark_goldenrod),
                           TextView('', 'back = ex_dark_goldenrod', back=Back.ex_dark_goldenrod)],
                          [TextView('', 'fore = ex_light_salmon_3a', width=30, fore=Fore.ex_light_salmon_3a),
                           TextView('', 'back = ex_light_salmon_3a', back=Back.ex_light_salmon_3a)],
                          [TextView('', 'fore = ex_rosy_brown', width=30, fore=Fore.ex_rosy_brown),
                           TextView('', 'back = ex_rosy_brown', back=Back.ex_rosy_brown)],
                          [TextView('', 'fore = ex_grey_63', width=30, fore=Fore.ex_grey_63),
                           TextView('', 'back = ex_grey_63', back=Back.ex_grey_63)],
                          [TextView('', 'fore = ex_medium_purple_2b', width=30, fore=Fore.ex_medium_purple_2b),
                           TextView('', 'back = ex_medium_purple_2b', back=Back.ex_medium_purple_2b)],
                          [TextView('', 'fore = ex_medium_purple_1', width=30, fore=Fore.ex_medium_purple_1),
                           TextView('', 'back = ex_medium_purple_1', back=Back.ex_medium_purple_1)],
                          [TextView('', 'fore = ex_gold_3a', width=30, fore=Fore.ex_gold_3a),
                           TextView('', 'back = ex_gold_3a', back=Back.ex_gold_3a)],
                          [TextView('', 'fore = ex_dark_khaki', width=30, fore=Fore.ex_dark_khaki),
                           TextView('', 'back = ex_dark_khaki', back=Back.ex_dark_khaki)],
                          [TextView('', 'fore = ex_navajo_white_3', width=30, fore=Fore.ex_navajo_white_3),
                           TextView('', 'back = ex_navajo_white_3', back=Back.ex_navajo_white_3)],
                          [TextView('', 'fore = ex_grey_69', width=30, fore=Fore.ex_grey_69),
                           TextView('', 'back = ex_grey_69', back=Back.ex_grey_69)],
                          [TextView('', 'fore = ex_light_steel_blue_3', width=30, fore=Fore.ex_light_steel_blue_3),
                           TextView('', 'back = ex_light_steel_blue_3', back=Back.ex_light_steel_blue_3)],
                          [TextView('', 'fore = ex_light_steel_blue', width=30, fore=Fore.ex_light_steel_blue),
                           TextView('', 'back = ex_light_steel_blue', back=Back.ex_light_steel_blue)],
                          [TextView('', 'fore = ex_yellow_3a', width=30, fore=Fore.ex_yellow_3a),
                           TextView('', 'back = ex_yellow_3a', back=Back.ex_yellow_3a)],
                          [TextView('', 'fore = ex_dark_olive_green_3', width=30, fore=Fore.ex_dark_olive_green_3),
                           TextView('', 'back = ex_dark_olive_green_3', back=Back.ex_dark_olive_green_3)],
                          [TextView('', 'fore = ex_dark_sea_green_3b', width=30, fore=Fore.ex_dark_sea_green_3b),
                           TextView('', 'back = ex_dark_sea_green_3b', back=Back.ex_dark_sea_green_3b)],
                          [TextView('', 'fore = ex_dark_sea_green_2', width=30, fore=Fore.ex_dark_sea_green_2),
                           TextView('', 'back = ex_dark_sea_green_2', back=Back.ex_dark_sea_green_2)],
                          [TextView('', 'fore = ex_light_cyan_3', width=30, fore=Fore.ex_light_cyan_3),
                           TextView('', 'back = ex_light_cyan_3', back=Back.ex_light_cyan_3)],
                          [TextView('', 'fore = ex_light_sky_blue_1', width=30, fore=Fore.ex_light_sky_blue_1),
                           TextView('', 'back = ex_light_sky_blue_1', back=Back.ex_light_sky_blue_1)],
                          [TextView('', 'fore = ex_green_yellow', width=30, fore=Fore.ex_green_yellow),
                           TextView('', 'back = ex_green_yellow', back=Back.ex_green_yellow)],
                          [TextView('', 'fore = ex_dark_olive_green_2', width=30, fore=Fore.ex_dark_olive_green_2),
                           TextView('', 'back = ex_dark_olive_green_2', back=Back.ex_dark_olive_green_2)],
                          [TextView('', 'fore = ex_pale_green_1b', width=30, fore=Fore.ex_pale_green_1b),
                           TextView('', 'back = ex_pale_green_1b', back=Back.ex_pale_green_1b)],
                          [TextView('', 'fore = ex_dark_sea_green_5b', width=30, fore=Fore.ex_dark_sea_green_5b),
                           TextView('', 'back = ex_dark_sea_green_5b', back=Back.ex_dark_sea_green_5b)],
                          [TextView('', 'fore = ex_dark_sea_green_5a', width=30, fore=Fore.ex_dark_sea_green_5a),
                           TextView('', 'back = ex_dark_sea_green_5a', back=Back.ex_dark_sea_green_5a)],
                          [TextView('', 'fore = ex_pale_turquoise_1', width=30, fore=Fore.ex_pale_turquoise_1),
                           TextView('', 'back = ex_pale_turquoise_1', back=Back.ex_pale_turquoise_1)],
                          [TextView('', 'fore = ex_red_3b', width=30, fore=Fore.ex_red_3b),
                           TextView('', 'back = ex_red_3b', back=Back.ex_red_3b)],
                          [TextView('', 'fore = ex_deep_pink_3a', width=30, fore=Fore.ex_deep_pink_3a),
                           TextView('', 'back = ex_deep_pink_3a', back=Back.ex_deep_pink_3a)],
                          [TextView('', 'fore = ex_deep_pink_3b', width=30, fore=Fore.ex_deep_pink_3b),
                           TextView('', 'back = ex_deep_pink_3b', back=Back.ex_deep_pink_3b)],
                          [TextView('', 'fore = ex_magenta_3b', width=30, fore=Fore.ex_magenta_3b),
                           TextView('', 'back = ex_magenta_3b', back=Back.ex_magenta_3b)],
                          [TextView('', 'fore = ex_magenta_3c', width=30, fore=Fore.ex_magenta_3c),
                           TextView('', 'back = ex_magenta_3c', back=Back.ex_magenta_3c)],
                          [TextView('', 'fore = ex_magenta_2a', width=30, fore=Fore.ex_magenta_2a),
                           TextView('', 'back = ex_magenta_2a', back=Back.ex_magenta_2a)],
                          [TextView('', 'fore = ex_dark_orange_3b', width=30, fore=Fore.ex_dark_orange_3b),
                           TextView('', 'back = ex_dark_orange_3b', back=Back.ex_dark_orange_3b)],
                          [TextView('', 'fore = ex_indian_red_1b', width=30, fore=Fore.ex_indian_red_1b),
                           TextView('', 'back = ex_indian_red_1b', back=Back.ex_indian_red_1b)],
                          [TextView('', 'fore = ex_hot_pink_3b', width=30, fore=Fore.ex_hot_pink_3b),
                           TextView('', 'back = ex_hot_pink_3b', back=Back.ex_hot_pink_3b)],
                          [TextView('', 'fore = ex_hot_pink_2', width=30, fore=Fore.ex_hot_pink_2),
                           TextView('', 'back = ex_hot_pink_2', back=Back.ex_hot_pink_2)],
                          [TextView('', 'fore = ex_orchid', width=30, fore=Fore.ex_orchid),
                           TextView('', 'back = ex_orchid', back=Back.ex_orchid)],
                          [TextView('', 'fore = ex_medium_orchid_1a', width=30, fore=Fore.ex_medium_orchid_1a),
                           TextView('', 'back = ex_medium_orchid_1a', back=Back.ex_medium_orchid_1a)],
                          [TextView('', 'fore = ex_orange_3', width=30, fore=Fore.ex_orange_3),
                           TextView('', 'back = ex_orange_3', back=Back.ex_orange_3)],
                          [TextView('', 'fore = ex_light_salmon_3b', width=30, fore=Fore.ex_light_salmon_3b),
                           TextView('', 'back = ex_light_salmon_3b', back=Back.ex_light_salmon_3b)],
                          [TextView('', 'fore = ex_light_pink_3', width=30, fore=Fore.ex_light_pink_3),
                           TextView('', 'back = ex_light_pink_3', back=Back.ex_light_pink_3)],
                          [TextView('', 'fore = ex_pink_3', width=30, fore=Fore.ex_pink_3),
                           TextView('', 'back = ex_pink_3', back=Back.ex_pink_3)],
                          [TextView('', 'fore = ex_plum_3', width=30, fore=Fore.ex_plum_3),
                           TextView('', 'back = ex_plum_3', back=Back.ex_plum_3)],
                          [TextView('', 'fore = ex_violet', width=30, fore=Fore.ex_violet),
                           TextView('', 'back = ex_violet', back=Back.ex_violet)],
                          [TextView('', 'fore = ex_gold_3b', width=30, fore=Fore.ex_gold_3b),
                           TextView('', 'back = ex_gold_3b', back=Back.ex_gold_3b)],
                          [TextView('', 'fore = ex_light_goldenrod_3', width=30, fore=Fore.ex_light_goldenrod_3),
                           TextView('', 'back = ex_light_goldenrod_3', back=Back.ex_light_goldenrod_3)],
                          [TextView('', 'fore = ex_tan', width=30, fore=Fore.ex_tan),
                           TextView('', 'back = ex_tan', back=Back.ex_tan)],
                          [TextView('', 'fore = ex_misty_rose_3', width=30, fore=Fore.ex_misty_rose_3),
                           TextView('', 'back = ex_misty_rose_3', back=Back.ex_misty_rose_3)],
                          [TextView('', 'fore = ex_thistle_3', width=30, fore=Fore.ex_thistle_3),
                           TextView('', 'back = ex_thistle_3', back=Back.ex_thistle_3)],
                          [TextView('', 'fore = ex_plum_2', width=30, fore=Fore.ex_plum_2),
                           TextView('', 'back = ex_plum_2', back=Back.ex_plum_2)],
                          [TextView('', 'fore = ex_yellow_3b', width=30, fore=Fore.ex_yellow_3b),
                           TextView('', 'back = ex_yellow_3b', back=Back.ex_yellow_3b)],
                          [TextView('', 'fore = ex_khaki_3', width=30, fore=Fore.ex_khaki_3),
                           TextView('', 'back = ex_khaki_3', back=Back.ex_khaki_3)],
                          [TextView('', 'fore = ex_light_goldenrod_2a', width=30, fore=Fore.ex_light_goldenrod_2a),
                           TextView('', 'back = ex_light_goldenrod_2a', back=Back.ex_light_goldenrod_2a)],
                          [TextView('', 'fore = ex_light_yellow_3', width=30, fore=Fore.ex_light_yellow_3),
                           TextView('', 'back = ex_light_yellow_3', back=Back.ex_light_yellow_3)],
                          [TextView('', 'fore = ex_grey_84', width=30, fore=Fore.ex_grey_84),
                           TextView('', 'back = ex_grey_84', back=Back.ex_grey_84)],
                          [TextView('', 'fore = ex_light_steel_blue_1', width=30, fore=Fore.ex_light_steel_blue_1),
                           TextView('', 'back = ex_light_steel_blue_1', back=Back.ex_light_steel_blue_1)],
                          [TextView('', 'fore = ex_yellow_2', width=30, fore=Fore.ex_yellow_2),
                           TextView('', 'back = ex_yellow_2', back=Back.ex_yellow_2)],
                          [TextView('', 'fore = ex_dark_olive_green_1a', width=30, fore=Fore.ex_dark_olive_green_1a),
                           TextView('', 'back = ex_dark_olive_green_1a', back=Back.ex_dark_olive_green_1a)],
                          [TextView('', 'fore = ex_dark_olive_green_1b', width=30, fore=Fore.ex_dark_olive_green_1b),
                           TextView('', 'back = ex_dark_olive_green_1b', back=Back.ex_dark_olive_green_1b)],
                          [TextView('', 'fore = ex_dark_sea_green_1', width=30, fore=Fore.ex_dark_sea_green_1),
                           TextView('', 'back = ex_dark_sea_green_1', back=Back.ex_dark_sea_green_1)],
                          [TextView('', 'fore = ex_honeydew_2', width=30, fore=Fore.ex_honeydew_2),
                           TextView('', 'back = ex_honeydew_2', back=Back.ex_honeydew_2)],
                          [TextView('', 'fore = ex_light_cyan_1', width=30, fore=Fore.ex_light_cyan_1),
                           TextView('', 'back = ex_light_cyan_1', back=Back.ex_light_cyan_1)],
                          [TextView('', 'fore = ex_red_1', width=30, fore=Fore.ex_red_1),
                           TextView('', 'back = ex_red_1', back=Back.ex_red_1)],
                          [TextView('', 'fore = ex_deep_pink_2', width=30, fore=Fore.ex_deep_pink_2),
                           TextView('', 'back = ex_deep_pink_2', back=Back.ex_deep_pink_2)],
                          [TextView('', 'fore = ex_deep_pink_1a', width=30, fore=Fore.ex_deep_pink_1a),
                           TextView('', 'back = ex_deep_pink_1a', back=Back.ex_deep_pink_1a)],
                          [TextView('', 'fore = ex_deep_pink_1b', width=30, fore=Fore.ex_deep_pink_1b),
                           TextView('', 'back = ex_deep_pink_1b', back=Back.ex_deep_pink_1b)],
                          [TextView('', 'fore = ex_magenta_2b', width=30, fore=Fore.ex_magenta_2b),
                           TextView('', 'back = ex_magenta_2b', back=Back.ex_magenta_2b)],
                          [TextView('', 'fore = ex_magenta_1', width=30, fore=Fore.ex_magenta_1),
                           TextView('', 'back = ex_magenta_1', back=Back.ex_magenta_1)],
                          [TextView('', 'fore = ex_orange_red_1', width=30, fore=Fore.ex_orange_red_1),
                           TextView('', 'back = ex_orange_red_1', back=Back.ex_orange_red_1)],
                          [TextView('', 'fore = ex_indian_red_1c', width=30, fore=Fore.ex_indian_red_1c),
                           TextView('', 'back = ex_indian_red_1c', back=Back.ex_indian_red_1c)],
                          [TextView('', 'fore = ex_indian_red_1d', width=30, fore=Fore.ex_indian_red_1d),
                           TextView('', 'back = ex_indian_red_1d', back=Back.ex_indian_red_1d)],
                          [TextView('', 'fore = ex_hot_pink_1a', width=30, fore=Fore.ex_hot_pink_1a),
                           TextView('', 'back = ex_hot_pink_1a', back=Back.ex_hot_pink_1a)],
                          [TextView('', 'fore = ex_hot_pink_1b', width=30, fore=Fore.ex_hot_pink_1b),
                           TextView('', 'back = ex_hot_pink_1b', back=Back.ex_hot_pink_1b)],
                          [TextView('', 'fore = ex_medium_orchid_1b', width=30, fore=Fore.ex_medium_orchid_1b),
                           TextView('', 'back = ex_medium_orchid_1b', back=Back.ex_medium_orchid_1b)],
                          [TextView('', 'fore = ex_dark_orange', width=30, fore=Fore.ex_dark_orange),
                           TextView('', 'back = ex_dark_orange', back=Back.ex_dark_orange)],
                          [TextView('', 'fore = ex_salmon_1', width=30, fore=Fore.ex_salmon_1),
                           TextView('', 'back = ex_salmon_1', back=Back.ex_salmon_1)],
                          [TextView('', 'fore = ex_light_coral', width=30, fore=Fore.ex_light_coral),
                           TextView('', 'back = ex_light_coral', back=Back.ex_light_coral)],
                          [TextView('', 'fore = ex_pale_violet_red_1', width=30, fore=Fore.ex_pale_violet_red_1),
                           TextView('', 'back = ex_pale_violet_red_1', back=Back.ex_pale_violet_red_1)],
                          [TextView('', 'fore = ex_orchid_2', width=30, fore=Fore.ex_orchid_2),
                           TextView('', 'back = ex_orchid_2', back=Back.ex_orchid_2)],
                          [TextView('', 'fore = ex_orchid_1', width=30, fore=Fore.ex_orchid_1),
                           TextView('', 'back = ex_orchid_1', back=Back.ex_orchid_1)],
                          [TextView('', 'fore = ex_orange_1', width=30, fore=Fore.ex_orange_1),
                           TextView('', 'back = ex_orange_1', back=Back.ex_orange_1)],
                          [TextView('', 'fore = ex_sandy_brown', width=30, fore=Fore.ex_sandy_brown),
                           TextView('', 'back = ex_sandy_brown', back=Back.ex_sandy_brown)],
                          [TextView('', 'fore = ex_light_salmon_1', width=30, fore=Fore.ex_light_salmon_1),
                           TextView('', 'back = ex_light_salmon_1', back=Back.ex_light_salmon_1)],
                          [TextView('', 'fore = ex_light_pink_1', width=30, fore=Fore.ex_light_pink_1),
                           TextView('', 'back = ex_light_pink_1', back=Back.ex_light_pink_1)],
                          [TextView('', 'fore = ex_pink_1', width=30, fore=Fore.ex_pink_1),
                           TextView('', 'back = ex_pink_1', back=Back.ex_pink_1)],
                          [TextView('', 'fore = ex_plum_1', width=30, fore=Fore.ex_plum_1),
                           TextView('', 'back = ex_plum_1', back=Back.ex_plum_1)],
                          [TextView('', 'fore = ex_gold_1', width=30, fore=Fore.ex_gold_1),
                           TextView('', 'back = ex_gold_1', back=Back.ex_gold_1)],
                          [TextView('', 'fore = ex_light_goldenrod_2b', width=30, fore=Fore.ex_light_goldenrod_2b),
                           TextView('', 'back = ex_light_goldenrod_2b', back=Back.ex_light_goldenrod_2b)],
                          [TextView('', 'fore = ex_light_goldenrod_2c', width=30, fore=Fore.ex_light_goldenrod_2c),
                           TextView('', 'back = ex_light_goldenrod_2c', back=Back.ex_light_goldenrod_2c)],
                          [TextView('', 'fore = ex_navajo_white_1', width=30, fore=Fore.ex_navajo_white_1),
                           TextView('', 'back = ex_navajo_white_1', back=Back.ex_navajo_white_1)],
                          [TextView('', 'fore = ex_misty_rose1', width=30, fore=Fore.ex_misty_rose1),
                           TextView('', 'back = ex_misty_rose1', back=Back.ex_misty_rose1)],
                          [TextView('', 'fore = ex_thistle_1', width=30, fore=Fore.ex_thistle_1),
                           TextView('', 'back = ex_thistle_1', back=Back.ex_thistle_1)],
                          [TextView('', 'fore = ex_yellow_1', width=30, fore=Fore.ex_yellow_1),
                           TextView('', 'back = ex_yellow_1', back=Back.ex_yellow_1)],
                          [TextView('', 'fore = ex_light_goldenrod_1', width=30, fore=Fore.ex_light_goldenrod_1),
                           TextView('', 'back = ex_light_goldenrod_1', back=Back.ex_light_goldenrod_1)],
                          [TextView('', 'fore = ex_khaki_1', width=30, fore=Fore.ex_khaki_1),
                           TextView('', 'back = ex_khaki_1', back=Back.ex_khaki_1)],
                          [TextView('', 'fore = ex_wheat_1', width=30, fore=Fore.ex_wheat_1),
                           TextView('', 'back = ex_wheat_1', back=Back.ex_wheat_1)],
                          [TextView('', 'fore = ex_cornsilk_1', width=30, fore=Fore.ex_cornsilk_1),
                           TextView('', 'back = ex_cornsilk_1', back=Back.ex_cornsilk_1)],
                          [TextView('', 'fore = ex_grey_100', width=30, fore=Fore.ex_grey_100),
                           TextView('', 'back = ex_grey_100', back=Back.ex_grey_100)],
                          [TextView('', 'fore = ex_grey_3', width=30, fore=Fore.ex_grey_3),
                           TextView('', 'back = ex_grey_3', back=Back.ex_grey_3)],
                          [TextView('', 'fore = ex_grey_7', width=30, fore=Fore.ex_grey_7),
                           TextView('', 'back = ex_grey_7', back=Back.ex_grey_7)],
                          [TextView('', 'fore = ex_grey_11', width=30, fore=Fore.ex_grey_11),
                           TextView('', 'back = ex_grey_11', back=Back.ex_grey_11)],
                          [TextView('', 'fore = ex_grey_15', width=30, fore=Fore.ex_grey_15),
                           TextView('', 'back = ex_grey_15', back=Back.ex_grey_15)],
                          [TextView('', 'fore = ex_grey_19', width=30, fore=Fore.ex_grey_19),
                           TextView('', 'back = ex_grey_19', back=Back.ex_grey_19)],
                          [TextView('', 'fore = ex_grey_23', width=30, fore=Fore.ex_grey_23),
                           TextView('', 'back = ex_grey_23', back=Back.ex_grey_23)],
                          [TextView('', 'fore = ex_grey_27', width=30, fore=Fore.ex_grey_27),
                           TextView('', 'back = ex_grey_27', back=Back.ex_grey_27)],
                          [TextView('', 'fore = ex_grey_30', width=30, fore=Fore.ex_grey_30),
                           TextView('', 'back = ex_grey_30', back=Back.ex_grey_30)],
                          [TextView('', 'fore = ex_grey_35', width=30, fore=Fore.ex_grey_35),
                           TextView('', 'back = ex_grey_35', back=Back.ex_grey_35)],
                          [TextView('', 'fore = ex_grey_39', width=30, fore=Fore.ex_grey_39),
                           TextView('', 'back = ex_grey_39', back=Back.ex_grey_39)],
                          [TextView('', 'fore = ex_grey_42', width=30, fore=Fore.ex_grey_42),
                           TextView('', 'back = ex_grey_42', back=Back.ex_grey_42)],
                          [TextView('', 'fore = ex_grey_46', width=30, fore=Fore.ex_grey_46),
                           TextView('', 'back = ex_grey_46', back=Back.ex_grey_46)],
                          [TextView('', 'fore = ex_grey_50', width=30, fore=Fore.ex_grey_50),
                           TextView('', 'back = ex_grey_50', back=Back.ex_grey_50)],
                          [TextView('', 'fore = ex_grey_54', width=30, fore=Fore.ex_grey_54),
                           TextView('', 'back = ex_grey_54', back=Back.ex_grey_54)],
                          [TextView('', 'fore = ex_grey_58', width=30, fore=Fore.ex_grey_58),
                           TextView('', 'back = ex_grey_58', back=Back.ex_grey_58)],
                          [TextView('', 'fore = ex_grey_62', width=30, fore=Fore.ex_grey_62),
                           TextView('', 'back = ex_grey_62', back=Back.ex_grey_62)],
                          [TextView('', 'fore = ex_grey_66', width=30, fore=Fore.ex_grey_66),
                           TextView('', 'back = ex_grey_66', back=Back.ex_grey_66)],
                          [TextView('', 'fore = ex_grey_70', width=30, fore=Fore.ex_grey_70),
                           TextView('', 'back = ex_grey_70', back=Back.ex_grey_70)],
                          [TextView('', 'fore = ex_grey_74', width=30, fore=Fore.ex_grey_74),
                           TextView('', 'back = ex_grey_74', back=Back.ex_grey_74)],
                          [TextView('', 'fore = ex_grey_78', width=30, fore=Fore.ex_grey_78),
                           TextView('', 'back = ex_grey_78', back=Back.ex_grey_78)],
                          [TextView('', 'fore = ex_grey_82', width=30, fore=Fore.ex_grey_82),
                           TextView('', 'back = ex_grey_82', back=Back.ex_grey_82)],
                          [TextView('', 'fore = ex_grey_85', width=30, fore=Fore.ex_grey_85),
                           TextView('', 'back = ex_grey_85', back=Back.ex_grey_85)],
                          [TextView('', 'fore = ex_grey_89', width=30, fore=Fore.ex_grey_89),
                           TextView('', 'back = ex_grey_89', back=Back.ex_grey_89)],
                          [TextView('', 'fore = ex_grey_93', width=30, fore=Fore.ex_grey_93),
                           TextView('', 'back = ex_grey_93', back=Back.ex_grey_93)]
                       ])

ctl.draw(auto_re_draw=False)



# -*- coding: utf-8 -*-
from terminal_layout import *

print('\ncolor support window, linux, osx\n')

ctl = LayoutCtl.quick(TableLayout,
                      [
                          [TextView('', 'fore = black', width=30, fore=Fore.black),
                           TextView('', 'back = black', back=Back.black)],
                          [TextView('', 'fore = red', width=30, fore=Fore.red), TextView('', 'back = red', back=Back.red)],
                          [TextView('', 'fore = green', width=30, fore=Fore.green),
                           TextView('', 'back = green', back=Back.green)],
                          [TextView('', 'fore = yellow', width=30, fore=Fore.yellow),
                           TextView('', 'back = yellow', back=Back.yellow)],
                          [TextView('', 'fore = blue', width=30, fore=Fore.blue), TextView('', 'back = blue', back=Back.blue)],
                          [TextView('', 'fore = magenta', width=30, fore=Fore.magenta),
                           TextView('', 'back = magenta', back=Back.magenta)],
                          [TextView('', 'fore = cyan', width=30, fore=Fore.cyan), TextView('', 'back = cyan', back=Back.cyan)],
                          [TextView('', 'fore = white', width=30, fore=Fore.white),
                           TextView('', 'back = white', back=Back.white)],
                          [TextView('', 'fore = lightblack', width=30, fore=Fore.lightblack),
                           TextView('', 'back = lightblack', back=Back.lightblack)],
                          [TextView('', 'fore = lightred', width=30, fore=Fore.lightred),
                           TextView('', 'back = lightred', back=Back.lightred)],
                          [TextView('', 'fore = lightgreen', width=30, fore=Fore.lightgreen),
                           TextView('', 'back = lightgreen', back=Back.lightgreen)],
                          [TextView('', 'fore = lightyellow', width=30, fore=Fore.lightyellow),
                           TextView('', 'back = lightyellow', back=Back.lightyellow)],
                          [TextView('', 'fore = lightblue', width=30, fore=Fore.lightblue),
                           TextView('', 'back = lightblue', back=Back.lightblue)],
                          [TextView('', 'fore = lightmagenta', width=30, fore=Fore.lightmagenta),
                           TextView('', 'back = lightmagenta', back=Back.lightmagenta)],
                          [TextView('', 'fore = lightcyan', width=30, fore=Fore.lightcyan),
                           TextView('', 'back = lightcyan', back=Back.lightcyan)],
                          [TextView('', 'fore = lightwhite', width=30, fore=Fore.lightwhite),
                           TextView('', 'back = lightwhite', back=Back.lightwhite)]
                      ])

ctl.draw(auto_re_draw=False)





# -*- coding: utf-8 -*-
from colorama.ansi import code_to_chars
from colored import fg, bg, attr


class Font(object):
    ex_function = {
        'fore': fg,
        'back': bg,
        'style': attr
    }

    def __init__(self, name, type, code=None):
        """

        :param name: color name or style name
        :param type: fore ,back , style
        :param code: colorama color,style num
        """
        self.name = name
        self.type = type
        self.code = code

    def __str__(self):
        if not self.name:
            return ''
        if self.name.startswith('ex_'):
            func = self.ex_function[self.type]
            return func(self.name[3:])
        else:
            return code_to_chars(self.code)

    def __add__(self, other):
        print('add')
        return str(self) + str(other)



# -*- coding: utf-8 -*-
from terminal_layout.ansi.font import Font


class Style(object):

    bright = Font('bright', 'style', 1)
    dim = Font('dim', 'style', 2)
    normal = Font('normal', 'style', 22)
    reset_all = Font('reset_all', 'style', 0)

    # ex_xx only support linux, mac

    ex_bold = Font('ex_bold', 'style')
    ex_dim = Font('ex_dim', 'style')
    ex_underlined = Font('ex_underlined', 'style')
    ex_blink = Font('ex_blink', 'style')
    ex_reverse = Font('ex_reverse', 'style')
    ex_hidden = Font('ex_hidden', 'style')
    ex_reset = Font('ex_reset', 'style')
    ex_res_bold = Font('ex_res_bold', 'style')
    ex_res_dim = Font('ex_res_dim', 'style')
    ex_res_underlined = Font('ex_res_underlined', 'style')
    ex_res_blink = Font('ex_res_blink', 'style')
    ex_res_reverse = Font('ex_res_reverse', 'style')
    ex_res_hidden = Font('ex_res_hidden', 'style')



import sys

from terminal_layout import *
from terminal_layout.helper.class_helper import instance_variables


class SuffixStyle:
    fraction = '{fraction}'  # "1/10"
    percent = '{percent}'  # "10%"
    none = ''


class ProgressWidth:
    fill = 'fill'
    half = 'half'  # half of terminal width


class Progress(object):
    ctl = None

    @instance_variables
    def __init__(self, prefix, max, reached=None, unreached='', delimiter=None, suffix_style=SuffixStyle.percent,
                 width=ProgressWidth.half):
        """

        :param prefix: 
        :type prefix: str
        :param max: 
        :type max: int
        :param reached: 
        :type reached: str
        :param unreached: 
        :type unreached: str
        :param suffix_style: 
        :type suffix_style: SuffixStyle
        :param width: ProgressWidth or width num. width=len(prefix)+len(progress)+len(delimiter)+len(suffix)
        """
        if delimiter is None:
            self.delimiter = [' |', '| ']
        assert self.max > 0
        self.current_progress = 0
        if self.reached is None:
            if sys.platform in ('win32', 'cygwin'):
                self.reached = '='
            else:
                self.reached = '█'

    def get_suffix(self):
        fraction = '%d/%d' % (self.current_progress, self.max)
        percent = '{:.0%}'.format(float(self.current_progress) / self.max)

        return self.suffix_style.format(fraction=fraction, percent=percent)

    def set_progress(self, num):
        if num > self.max:
            num = self.max
        elif num < 0:
            num = 0
        self.current_progress = num
        self.update()

    def add_progress(self, num):
        if num < 0:
            num = 0
        self.current_progress += num
        if self.current_progress > self.max:
            self.current_progress = self.max
        self.update()

    def update(self):
        progress_view = self.ctl.find_view_by_id('progress')
        w = progress_view.get_real_width()
        reached_num = int(float(self.current_progress) / self.max * w)
        s = self.reached * reached_num + self.unreached * (w - reached_num)
        progress_view.set_text(s)

        self.ctl.find_view_by_id('suffix').set_text(self.get_suffix())

    def start(self):
        prefix_width = len(self.prefix)
        delimiter_width = len(self.delimiter[0]) + len(self.delimiter[1])
        if self.suffix_style == SuffixStyle.percent:
            suffix_width = 4
        else:
            suffix_width = len(str(self.max)) * 2 + 1

        self.ctl = LayoutCtl.quick(TableLayout,
                                   [[TextView('prefix', self.prefix, width=prefix_width),
                                     TextView('', self.delimiter[0]),
                                     TextView('progress', ''),
                                     TextView('', self.delimiter[1]),
                                     TextView('suffix', self.get_suffix(), width=suffix_width, gravity=Gravity.right)]])

        if self.width == ProgressWidth.fill:
            self.ctl.find_view_by_id('progress').set_weight(1)
        else:
            terminal_width, _ = self.ctl.get_terminal_size()
            if self.width == ProgressWidth.half:
                progress_width = int((terminal_width - prefix_width - suffix_width - delimiter_width) / 2)
            else:
                progress_width = self.width - prefix_width - suffix_width - delimiter_width
            self.ctl.find_view_by_id('progress').set_width(progress_width)
        self.ctl.update_width()
        self.update()
        self.ctl.draw()

    def stop(self):
        if self.ctl:
            self.ctl.stop()
        self.ctl = None

    def is_finished(self):
        return self.current_progress >= self.max

    def __enter__(self):
        self.start()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.stop()



# -*- coding: utf-8 -*-

__author__ = 'gojuukaze'

from setuptools import setup, find_packages

setup(
    name="terminal_layout",
    version="2.0.4",
    description="The project help you to quickly build layouts in terminal (命令行ui布局工具)",
    long_description=open("README.rst", encoding='utf-8').read(),

    url="https://github.com/gojuukaze/terminal_layout",
    author="gojuukaze",
    author_email="ikaze_email@163.com",

    packages=find_packages(exclude=['demo', 'tests']),
    python_requires='>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*',
    include_package_data=True,
    platforms="OSX, Linux",
    install_requires=['colorama==0.4.4',
                      'colored==1.3.93',
                      'backports.shutil_get_terminal_size'],
    tests_require=['colorama==0.4.4',
                   'colored==1.3.93'],

    classifiers=[
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Development Status :: 5 - Production/Stable',
        'Topic :: Terminals',
    ],

    project_urls={
        'Documentation': 'https://doc.ikaze.cn/terminal_layout',
        'Source': 'https://github.com/gojuukaze/terminal_layout',
    },
)



# -*- coding: utf-8 -*-
import sys
import threading

from terminal_layout.ansi import *
from terminal_layout.types import String
from terminal_layout.view.base import View
from terminal_layout.view.params import Visibility, Width, Gravity


class TextView(View):
    __slots__ = ('back', 'style', 'fore', 'text', 'text_string', 'weight')

    def __init__(self, id, text, fore=None, back=None, style=None, width=Width.wrap,
                 height=1, weight=None, visibility=Visibility.visible, gravity=Gravity.left):
        """

        :param id:
        :param width:
        :param height: no used

        :type id:str
        :type width:
        :type height:int
        :type visibility:str
        :type gravity:str
        """

        super(TextView, self).__init__(id, width, height, visibility, gravity)
        self.text_string = None  # type:String

        self.text = text
        self.fore = fore or ''
        self.back = back or ''
        self.style = style or ''
        self.weight = weight

        self.real_height = 1

    def update_width(self, parent_width):
        if parent_width <= 0:
            self.real_width = 0
            return self.real_width
        if self.weight:
            return None
        if isinstance(self.width, int) and self.width >= 0:
            self.real_width = int(self.width)
            temp = parent_width - self.real_width
            if temp <= 0:
                self.real_width = parent_width

        elif self.width == Width.wrap:
            self.real_width = len(self.text_string)
            temp = parent_width - self.real_width
            if temp <= 0:
                self.real_width = parent_width

        elif self.width == Width.fill:
            self.real_width = parent_width

        return self.real_width

    def draw(self):
        sys.stdout.write(self.get_final_text())

    def clear(self):
        sys.stdout.write(Cursor.UP(self.real_height) + clear_line())

    def get_final_text(self):

        if self.visibility == Visibility.visible:
            self.real_height = 1
            show_text = self.text_string[:self.real_width]
        elif self.visibility == Visibility.invisible:
            self.real_height = 1
            return ' ' * self.real_width
        elif self.visibility == Visibility.gone:
            return ''

        # show_text是str 要转为String
        show_string = String(show_text)
        if self.real_width > len(show_string):
            if self.gravity == Gravity.left:
                show_text = show_text + ' ' * (self.real_width - len(show_string))
            elif self.gravity == Gravity.right:
                show_text = ' ' * (self.real_width - len(show_string)) + show_text
            elif self.gravity == Gravity.center:
                p = (self.real_width - len(show_string)) / 2.0
                show_text = ' ' * int(p) + show_text + ' ' * (int(p) if int(p) == p else int(p) + 1)

        return str(self.fore) + str(self.back) + str(self.style) + show_text + str(Style.reset_all)

    def __str__(self):
        return str(self.fore) + str(self.back) + str(self.style) + self.text + str(Style.reset_all)

    def __setattr__(self, key, value):
        super(TextView, self).__setattr__(key, value)
        if key == 'text':
            self.text_string = String(value)


class InputView(TextView):
    """
    还需完善，暂时别用
    """
    is_focus = False

    def __init__(self, id, prompt=None, default=None, fore=None, back=None, style=None, width=Width.wrap,
                 height=1, weight=None, visibility=Visibility.visible, gravity=Gravity.left):
        """

        :param id:
        :param width:
        :param height: no used

        :type id:str
        :type width:
        :type height:int
        :type visibility:str
        :type gravity:str
        """

        super(InputView, self).__init__(id, '', fore=fore, back=back, style=style, width=width,
                                        height=height, weight=weight, visibility=visibility, gravity=gravity)

        self.prompt = prompt or ''
        self.default = default or ''

    def set_focus(self, is_focus):
        self.is_focus = is_focus
        if is_focus:
            self.t = threading.Thread()

    def get_input_text(self):
        """
        :return:
        :rtype: str
        """
        return self.text or self.default

    def __str__(self):
        return str(self.fore) + str(self.back) + str(self.style) + self.prompt + self.text + str(Style.reset_all)

    def __setattr__(self, key, value):
        super(TextView, self).__setattr__(key, value)

        if key == 'text':
            self.text_string = String(self.prompt + self.text)



import re
import sys
from datetime import datetime

from terminal_layout.readkey.key import Key, KeyInfo
from terminal_layout.readkey.event import KeyPressEvent

if sys.platform in ('win32', 'cygwin'):
    from terminal_layout.readkey.windows import readkey as _readkey
else:
    from terminal_layout.readkey.linux import readkey as _readkey


def get_code_to_name():
    """
    :return:
    :rtype: dict
    """
    d = {}
    for k, info in Key.__dict__.items():
        if not k.startswith("_"):
            d[info.code] = info.name
    return d


code_to_name = get_code_to_name()


def readkey():
    c = _readkey()
    n = datetime.now()
    name = code_to_name.get(c, None)
    if name:
        return KeyPressEvent(KeyInfo(name, c), n)
    if repr(c).startswith("'\\x") or repr(c).startswith("u'\\x"):
        return KeyPressEvent(KeyInfo('unknown', c), n)
    else:
        return KeyPressEvent(KeyInfo(c, c), n)


class KeyListener(object):

    def __init__(self):
        self.key_func = {}
        self.re_func = {}
        self.stop_flag = False

    def bind_key(self, *args, **kwargs):
        """
        args: key or regular expression;
        :param args:
        :param kwargs:
        """
        decorator = kwargs.get('decorator', True)
        if decorator:
            keys = args
        else:
            keys = args[:-1]

        def inner(func):
            for k in keys:
                if isinstance(k, KeyInfo):
                    self.key_func[k.code] = func
                elif isinstance(k, str):
                    self.re_func[k] = func
                else:
                    raise TypeError('item in *args must be a KeyInfo or a regular expression string')
            return func

        if decorator:
            return inner
        else:
            inner(args[-1])

    def stop(self):
        self.stop_flag = True

    def listen(self, stop_key=None):
        """

        :param stop_key: key or regular expression; default [Key.ESC]
        :type stop_key: list
        :return: 
        :rtype: 
        """
        if stop_key is None:
            stop_key = [Key.ESC]

        while True:
            if self.stop_flag:
                break
            c = readkey()
            code = c.key.code
            for s in stop_key:
                if isinstance(s, KeyInfo):
                    if s.code == code:
                        return
                else:
                    if re.match(s, code):
                        return

            func = self.key_func.get(c.key.code, None)
            if func:
                func(self, c)

            for s, func in self.re_func.items():
                if s == 'any':
                    func(self, c)
                    continue
                if re.match(s, code):
                    func(self, c)
                    continue



# -*- coding: utf-8 -*-
from terminal_layout.view.params import Visibility, Gravity


class View(object):
    __slots__ = ('id', 'width', 'height', 'visibility', 'gravity',
                 'real_width', 'real_height', 'data')

    def __init__(self, id, width, height=1, visibility=Visibility.visible, gravity=Gravity.left):
        """

        :param id:
        :param width:
        :param height: no used

        :type id:str
        :type width:
        :type height:int
        :type visibility:str
        :type gravity:str
        """

        self.data = []  # type: list[View]
        self.id = id
        self.width = width
        self.height = height
        self.visibility = visibility
        self.gravity = gravity

        self.real_width = None
        self.real_height = None

    def draw(self):
        pass

    def clear(self):
        pass

    def update_width(self, parent_width):
        pass

    def find_view_by_id(self, id):
        """

        :param id:
        :return:
        :rtype: View
        """
        if self.id == id:
            return self
        temp = None
        for v in self.data:
            temp = v.find_view_by_id(id)
            if temp:
                break
        return temp

    def get_width(self):
        """
        the width you set in __init__()
        初始化时设置的宽度

        :return:
        """
        return self.width

    def get_real_width(self):
        """

        the real width of the display
        实际显示的宽度

        :return:
        """
        return self.real_width

    def __getitem__(self, item):
        """

        :param item:
        :return:
        :rtype:View
        """
        return self.data[item]

    def insert(self, index, view):
        self.data.insert(index, view)



class KeyInfo:
    def __init__(self, name, code):
        self.name = name
        self.code = code

    def __eq__(self, other):
        if isinstance(other, KeyInfo):
            return self.code == other.code
        return self.code == other

    def __str__(self):
        return '<%s>' % self.name
    def __repr__(self):
        return self.__str__()

class Key:
    ENTER = KeyInfo('enter', '\r')
    TAB = KeyInfo('tab', '\x09')
    BACKSPACE = KeyInfo('backspace', '\x7f')
    ESC = KeyInfo('esc', '\x1b')

    UP = KeyInfo('up', '\x1b[A')
    DOWN = KeyInfo('down', '\x1b[B')
    LEFT = KeyInfo('left', '\x1b[D')
    RIGHT = KeyInfo('right', '\x1b[C')

    CTRL_A = KeyInfo('ctrl_a', '\x01')
    CTRL_B = KeyInfo('ctrl_b', '\x02')
    CTRL_C = KeyInfo('ctrl_c', '\x03')
    CTRL_D = KeyInfo('ctrl_d', '\x04')
    CTRL_E = KeyInfo('ctrl_e', '\x05')
    CTRL_F = KeyInfo('ctrl_f', '\x06')
    CTRL_X = KeyInfo('ctrl_x', '\x18')
    CTRL_Z = KeyInfo('ctrl_z', '\x1a')

    F1 = KeyInfo('f1', '\x1bOP')
    F2 = KeyInfo('f2', '\x1bOQ')
    F3 = KeyInfo('f3', '\x1bOR')
    F4 = KeyInfo('f4', '\x1bOS')
    F5 = KeyInfo('f5', '\x1b[15')
    F6 = KeyInfo('f6', '\x1b[17')
    F7 = KeyInfo('f7', '\x1b[18')
    F8 = KeyInfo('f8', '\x1b[19')



# -*- coding: utf-8 -*-
import os
import sys
import platform
import threading
import time

try:
    # py3
    from os import get_terminal_size
except:
    # py2
    from backports.shutil_get_terminal_size import get_terminal_size

from terminal_layout.ansi import term_init
from terminal_layout.view import *
from terminal_layout.view.base import View


class LayoutCtl(object):
    debug = False
    version = 0
    _drawing = False
    _stop_flag = False

    def __init__(self, layout=None):
        self.layout = layout  # type:View
        self.refresh_lock = threading.Lock()
        self.refresh_thread = threading.Thread(
            target=LayoutCtl.refresh,
            args=(self,),
        )
        self.refresh_thread.daemon = True

    def set_buffer_size(self, size):
        """
        当输出的文本太大会出现界面闪烁的情况，这时要调大sys.stdout的缓冲区
        （见 https://github.com/gojuukaze/terminal_layout/issues/3 ）
        建议在draw之前调用
        """
        self.buffering = size
        sys.stdout = os.fdopen(sys.stdout.fileno(), 'w', self.buffering, encoding='utf-8')

    def is_stop(self):
        return self._stop_flag

    @classmethod
    def quick(cls, layout_class, data):
        """

        :param layout_class:
        :param data:
        :return:
        :rtype :LayoutCtl
        """
        if layout_class is TableLayout:

            table_layout = TableLayout('root', Width.fill)
            i = 0
            for row_data in data:
                table_row = TableRow.quick_init('root_row_' + str(i), row_data, width=Width.fill)
                table_layout.add_view(table_row)
                i += 1
            return cls(table_layout)
        elif layout_class is TableRow:
            row = TableRow.quick_init('root', data, width=Width.fill)
            return cls(row)
        else:
            raise TypeError("quick not support %s" % (str(layout_class, )))

    def set_layout(self, layout):
        """

        :return:
        """

        self.layout = layout

    def get_layout(self):
        """

        :return:
        :rtype:LayoutProxy
        """
        return LayoutProxy(self, self.layout)

    def get_terminal_size(self):
        if self.debug:
            self.height = 10
            self.width = 50
            return

        size = get_terminal_size()
        self.height = size.lines
        self.width = size.columns

        if platform.system() == 'Windows':
            self.width -= 1
        return self.width, self.height

    def update_width(self):
        self.get_terminal_size()
        self.layout.update_width(self.width)

    def draw(self, auto_re_draw=True):
        term_init()
        self.version += 1
        self.update_width()
        self.layout.draw()

        sys.stdout.write('\n')
        sys.stdout.flush()
        if auto_re_draw:
            self.refresh_thread.start()

    def clear(self):
        self.layout.clear()

    def re_draw(self):
        self.refresh_lock.acquire()
        self.clear()

        self.update_width()
        self.layout.draw()

        sys.stdout.write('\n')
        sys.stdout.flush()
        self.refresh_lock.release()

    @staticmethod
    def refresh(ctl):
        while True:
            if ctl.is_stop():
                break
            time.sleep(0.1)
            ctl.re_draw()

    def find_view_by_id(self, id):
        v = self.layout.find_view_by_id(id)
        if not v:
            return None
        if isinstance(v, TextView):
            return TextViewProxy(self, v)
        else:
            return LayoutProxy(self, v)

    def stop(self):
        if not self.is_stop():
            self._stop_flag = True
            self.refresh_thread.join()
            self.re_draw()


class NULL:
    pass


class BaseViewProxy(object):
    def __init__(self, ctl, view):
        self.ctl = ctl  # type: LayoutCtl
        self.view = view

    def set(self, k, v, raise_error):
        try:
            setattr(self.view, k, v)
        except Exception as e:
            if raise_error:
                raise e

    def set_width(self, width, raise_error=False):
        self.set('width', width, raise_error)

    def set_visibility(self, visibility, raise_error=False):
        self.set('visibility', visibility, raise_error)

    def set_gravity(self, gravity, raise_error=False):
        self.set('gravity', gravity, raise_error)

    def set_text(self, text, raise_error=False):
        self.set('text', text, raise_error)

    def set_back(self, back, raise_error=False):
        self.set('back', back, raise_error)

    def set_style(self, style, raise_error=False):
        self.set('style', style, raise_error)

    def set_fore(self, fore, raise_error=False):
        self.set('fore', fore, raise_error)

    def set_weight(self, weight, raise_error=False):
        self.set('weight', weight, raise_error)

    def delay_set_text(self, text, delay=0.3):
        """
        set text one by one
        """
        s = ''
        for c in text:
            s += c
            self.set_text(s, raise_error=True)
            time.sleep(delay)

    def get(self, k, default):
        """
        When default == NULL , it is raised an error when the attribute doesn't exist
        如果default为NULL，当不存在变量时会抛错
        """
        if default == NULL:
            return getattr(self.view, k)
        else:
            return getattr(self.view, k, default)

    def get_id(self, default=NULL):
        return self.get('id', default)

    def get_width(self, default=NULL):
        return self.get('width', default)

    def get_real_width(self, default=NULL):
        return self.get('real_width', default)

    def get_visibility(self, default=NULL):
        return self.get('visibility', default)

    def get_gravity(self, default=NULL):
        return self.get('gravity', default)

    def get_text(self, default=NULL):
        return self.get('text', default)

    def get_back(self, default=NULL):
        return self.get('back', default)

    def get_style(self, default=NULL):
        self.get('style', default)

    def get_fore(self, default=NULL):
        self.get('fore', default)

    def get_weight(self, default=NULL):
        self.get('weight', default)


class TextViewProxy(BaseViewProxy):

    def set_text(self, text, raise_error=False):
        self.set('text', text, raise_error)

    def set_back(self, back, raise_error=False):
        self.set('back', back, raise_error)

    def set_style(self, style, raise_error=False):
        self.set('style', style, raise_error)

    def set_fore(self, fore, raise_error=False):
        self.set('fore', fore, raise_error)

    def set_weight(self, weight, raise_error=False):
        self.set('weight', weight, raise_error)

    def delay_set_text(self, text, delay=0.3):
        """
        set text one by one
        """
        s = ''
        for c in text:
            s += c
            self.set_text(s, raise_error=True)
            time.sleep(delay)

    def get_text(self, default=NULL):
        return self.get('text', default)

    def get_back(self, default=NULL):
        return self.get('back', default)

    def get_style(self, default=NULL):
        self.get('style', default)

    def get_fore(self, default=NULL):
        self.get('fore', default)

    def get_weight(self, default=NULL):
        self.get('weight', default)


class LayoutProxy(BaseViewProxy):
    def add_view(self, v):
        self.view.add_view(v)

    def add_views(self, *views):
        self.view.add_views(*views)

    def add_view_list(self, views):
        self.view.add_view_list(views)

    def insert_view(self, i, view):
        self.view.insert(i, view)
